@@ -5,6 +5,7 @@
 #pragma once
 
 #include "util/string.h"
+#include "util/unique_ptr.h"
 #include "util/vector.h"
 
 CCL_NAMESPACE_BEGIN
@@ -15,8 +16,12 @@ class Profiler;
 class Stats;
 
 bool device_metal_init();
+void device_metal_exit();
 
-Device *device_metal_create(const DeviceInfo &info, Stats &stats, Profiler &profiler);
+unique_ptr<Device> device_metal_create(const DeviceInfo &info,
+                                       Stats &stats,
+                                       Profiler &profiler,
+                                       bool headless);
 
 void device_metal_info(vector<DeviceInfo> &devices);
 
