@@ -2,7 +2,7 @@
  *
  * SPDX-License-Identifier: Apache-2.0 */
 
-#include "testing/testing.h"
+#include <gtest/gtest.h>
 
 #include "util/aligned_malloc.h"
 
@@ -14,7 +14,7 @@ TEST(util_aligned_malloc, aligned_malloc_16)
 {
   int *mem = (int *)util_aligned_malloc(sizeof(int), 16);
   CHECK_ALIGNMENT(mem, 16);
-  util_aligned_free(mem);
+  util_aligned_free(mem, sizeof(int));
 }
 
 /* On Apple we currently only support 16 bytes alignment. */
@@ -23,7 +23,7 @@ TEST(util_aligned_malloc, aligned_malloc_32)
 {
   int *mem = (int *)util_aligned_malloc(sizeof(int), 32);
   CHECK_ALIGNMENT(mem, 32);
-  util_aligned_free(mem);
+  util_aligned_free(mem, sizeof(int));
 }
 #endif /* __APPLE__ */
 
