@@ -20,6 +20,7 @@
 #include "BLI_string_utf8.h"
 
 #include "BLI_array.hh"
+#include "BLI_hash.h"
 #include "BLI_math_geom.h"
 #include "BLI_math_matrix.h"
 #include "BLI_math_rotation.h"
@@ -28,7 +29,7 @@
 #include "BLI_set.hh"
 #include "BLI_span.hh"
 #include "BLI_string_ref.hh"
-#include "BLI_vector.hh"
+#include "BLI_vector_list.hh"
 
 #include "DNA_collection_types.h"
 #include "DNA_curves_types.h"
@@ -38,6 +39,7 @@
 #include "DNA_pointcloud_types.h"
 #include "DNA_scene_types.h"
 #include "DNA_volume_types.h"
+#include "DNA_world_types.h"
 
 #include "BKE_collection.hh"
 #include "BKE_duplilist.hh"
@@ -58,10 +60,8 @@
 #include "DEG_depsgraph.hh"
 #include "DEG_depsgraph_query.hh"
 
-#include "BLI_hash.h"
-#include "DNA_world_types.h"
-
 #include "NOD_geometry_nodes_log.hh"
+
 #include "RNA_access.hh"
 #include "RNA_path.hh"
 #include "RNA_prototypes.hh"
@@ -75,6 +75,7 @@ using blender::float4x4;
 using blender::Set;
 using blender::Span;
 using blender::Vector;
+using blender::VectorList;
 using blender::bke::GeometrySet;
 using blender::bke::InstanceReference;
 using blender::bke::Instances;
@@ -137,7 +138,7 @@ struct DupliContext {
   const struct DupliGenerator *gen;
 
   /** Result containers. */
-  ListBase *duplilist; /* Legacy doubly-linked list. */
+  DupliList *duplilist;
 };
 
 struct DupliGenerator {
@@ -158,7 +159,8 @@ static void init_context(DupliContext *r_ctx,
                          const float space_mat[4][4],
                          blender::Set<const Object *> *include_objects,
                          Vector<Object *> &instance_stack,
-                         Vector<short> &dupli_gen_type_stack)
+                         Vector<short> &dupli_gen_type_stack,
+                         DupliList &duplilist)
 {
   r_ctx->depsgraph = depsgraph;
   r_ctx->scene = scene;
@@ -169,6 +171,7 @@ static void init_context(DupliContext *r_ctx,
   r_ctx->obedit = OBEDIT_FROM_OBACT(ob);
   r_ctx->instance_stack = &instance_stack;
   r_ctx->dupli_gen_type_stack = &dupli_gen_type_stack;
+  r_ctx->duplilist = &duplilist;
   if (space_mat) {
     copy_m4_m4(r_ctx->space_mat, space_mat);
   }
@@ -182,7 +185,6 @@ static void init_context(DupliContext *r_ctx,
     r_ctx->dupli_gen_type_stack->append(r_ctx->gen->type);
   }
 
-  r_ctx->duplilist = nullptr;
   r_ctx->preview_instance_index = -1;
   r_ctx->preview_base_geometry = nullptr;
 
@@ -211,7 +213,7 @@ static bool copy_dupli_context(DupliContext *r_ctx,
   r_ctx->object = ob;
   r_ctx->instance_stack = ctx->instance_stack;
   if (mat) {
-    mul_m4_m4m4(r_ctx->space_mat, (float(*)[4])ctx->space_mat, mat);
+    mul_m4_m4m4(r_ctx->space_mat, (float (*)[4])ctx->space_mat, mat);
   }
   r_ctx->persistent_id[r_ctx->level] = index;
   r_ctx->instance_idx[r_ctx->level] = instance_index;
@@ -268,8 +270,8 @@ static DupliObject *make_dupli(const DupliContext *ctx,
 
   /* Add a #DupliObject instance to the result container. */
   if (ctx->duplilist) {
-    dob = MEM_callocN<DupliObject>("dupli object");
-    BLI_addtail(ctx->duplilist, dob);
+    ctx->duplilist->append({});
+    dob = &ctx->duplilist->last();
   }
   else {
     return nullptr;
@@ -277,7 +279,7 @@ static DupliObject *make_dupli(const DupliContext *ctx,
 
   dob->ob = ob;
   dob->ob_data = const_cast<ID *>(object_data);
-  mul_m4_m4m4(dob->mat, (float(*)[4])ctx->space_mat, mat);
+  mul_m4_m4m4(dob->mat, (float (*)[4])ctx->space_mat, mat);
   dob->type = ctx->gen == nullptr ? 0 : ctx->dupli_gen_type_stack->last();
   dob->preview_base_geometry = ctx->preview_base_geometry;
   dob->preview_instance_index = ctx->preview_instance_index;
@@ -326,7 +328,7 @@ static DupliObject *make_dupli(const DupliContext *ctx,
   dob->random_id = BLI_hash_string(dob->ob->id.name + 2);
 
   if (dob->persistent_id[0] != INT_MAX) {
-    for (i = 0; i < MAX_DUPLI_RECUR; i++) {
+    for (i = 0; i < ctx->level + 1; i++) {
       dob->random_id = BLI_hash_int_2d(dob->random_id, uint(dob->persistent_id[i]));
     }
   }
@@ -777,7 +779,7 @@ static void make_duplis_verts(const DupliContext *ctx)
     vdd.totvert = mesh_eval->verts_num;
     vdd.vert_positions = mesh_eval->vert_positions();
     vdd.vert_normals = mesh_eval->vert_normals();
-    vdd.orco = (const float(*)[3])CustomData_get_layer(&mesh_eval->vert_data, CD_ORCO);
+    vdd.orco = (const float (*)[3])CustomData_get_layer(&mesh_eval->vert_data, CD_ORCO);
 
     make_child_duplis(ctx, &vdd, make_child_duplis_verts_from_mesh);
   }
@@ -825,37 +827,35 @@ static Object *find_family_object(
 
 static void make_duplis_font(const DupliContext *ctx)
 {
-  Object *par = ctx->object;
-  GHash *family_gh;
-  Object *ob;
-  Curve *cu;
-  CharTrans *ct, *chartransdata = nullptr;
-  float vec[3], obmat[4][4], pmat[4][4], fsize, xof, yof;
-  int text_len, a;
-  size_t family_len;
-  const char32_t *text = nullptr;
-  bool text_free = false;
-
   /* Font dupli-verts not supported inside collections. */
   if (ctx->collection) {
     return;
   }
 
+  Object *par = ctx->object;
+  GHash *family_gh;
+  Object *ob;
+  Curve *cu = (Curve *)par->data;
+  CharTrans *ct, *chartransdata = nullptr;
+  float vec[3], obmat[4][4], pmat[4][4];
+  int text_len, a;
+  size_t family_len;
+  const char32_t *text = nullptr;
+  bool text_free = false;
+
   copy_m4_m4(pmat, par->object_to_world().ptr());
 
   /* In `par` the family name is stored, use this to find the other objects. */
 
   BKE_vfont_to_curve_ex(
-      par, (Curve *)par->data, FO_DUPLI, nullptr, &text, &text_len, &text_free, &chartransdata);
+      par, *cu, FO_DUPLI, nullptr, &text, &text_len, &text_free, &chartransdata, nullptr);
 
   if (text == nullptr || chartransdata == nullptr) {
     return;
   }
 
-  cu = (Curve *)par->data;
-  fsize = cu->fsize;
-  xof = cu->xof;
-  yof = cu->yof;
+  const float fsize = cu->fsize;
+  const blender::float2 cu_offset = {cu->xof, cu->yof};
 
   ct = chartransdata;
 
@@ -880,19 +880,19 @@ static void make_duplis_font(const DupliContext *ctx)
     }
 
     if (ob) {
-      vec[0] = fsize * (ct->xof - xof);
-      vec[1] = fsize * (ct->yof - yof);
+      vec[0] = fsize * (ct->offset.x - cu_offset.x);
+      vec[1] = fsize * (ct->offset.y - cu_offset.y);
       vec[2] = 0.0;
 
       mul_m4_v3(pmat, vec);
 
       copy_m4_m4(obmat, par->object_to_world().ptr());
 
-      if (UNLIKELY(ct->rot != 0.0f)) {
+      if (UNLIKELY(ct->rotate != 0.0f)) {
         float rmat[4][4];
 
         zero_v3(obmat[3]);
-        axis_angle_to_mat4_single(rmat, 'Z', -ct->rot);
+        axis_angle_to_mat4_single(rmat, 'Z', -ct->rotate);
         mul_m4_m4m4(obmat, obmat, rmat);
       }
 
@@ -984,12 +984,12 @@ static void make_duplis_geometry_set_impl(const DupliContext *ctx,
 
   Span<float4x4> instance_offset_matrices = instances->transforms();
   Span<int> reference_handles = instances->reference_handles();
-  Span<int> almost_unique_ids = instances->almost_unique_ids();
+  Span<int> unique_ids = instances->unique_ids();
   Span<InstanceReference> references = instances->references();
 
   for (int64_t i : instance_offset_matrices.index_range()) {
     const InstanceReference &reference = references[reference_handles[i]];
-    const int id = almost_unique_ids[i];
+    const int id = unique_ids[i];
 
     const DupliContext *ctx_for_instance = instances_ctx;
     /* Set the #preview_instance_index when necessary. */
@@ -1112,7 +1112,7 @@ struct FaceDupliData_Mesh {
   Span<int> corner_verts;
   Span<float3> vert_positions;
   const float (*orco)[3];
-  const float2 *mloopuv;
+  const float2 *uv_map;
 };
 
 struct FaceDupliData_EditMesh {
@@ -1149,7 +1149,7 @@ static void get_dupliface_transform_from_coords(Span<float3> coords,
   /* Scale. */
   float scale;
   if (use_scale) {
-    const float area = area_poly_v3((const float(*)[3])coords.data(), uint(coords.size()));
+    const float area = area_poly_v3((const float (*)[3])coords.data(), uint(coords.size()));
     scale = sqrtf(area) * scale_fac;
   }
   else {
@@ -1255,8 +1255,8 @@ static void make_child_duplis_faces_from_mesh(const DupliContext *ctx,
                                               Object *inst_ob)
 {
   FaceDupliData_Mesh *fdd = (FaceDupliData_Mesh *)userdata;
-  const float(*orco)[3] = fdd->orco;
-  const float2 *mloopuv = fdd->mloopuv;
+  const float (*orco)[3] = fdd->orco;
+  const float2 *uv_map = fdd->uv_map;
   const int totface = fdd->totface;
   const bool use_scale = fdd->params.use_scale;
 
@@ -1285,9 +1285,9 @@ static void make_child_duplis_faces_from_mesh(const DupliContext *ctx,
         madd_v3_v3fl(dob->orco, orco[face_verts[j]], w);
       }
     }
-    if (mloopuv) {
+    if (uv_map) {
       for (int j = 0; j < face.size(); j++) {
-        madd_v2_v2fl(dob->uv, mloopuv[face[j]], w);
+        madd_v2_v2fl(dob->uv, uv_map[face[j]], w);
       }
     }
   }
@@ -1368,10 +1368,10 @@ static void make_duplis_faces(const DupliContext *ctx)
     fdd.faces = mesh_eval->faces();
     fdd.corner_verts = mesh_eval->corner_verts();
     fdd.vert_positions = mesh_eval->vert_positions();
-    fdd.mloopuv = (uv_idx != -1) ? (const float2 *)CustomData_get_layer_n(
-                                       &mesh_eval->corner_data, CD_PROP_FLOAT2, uv_idx) :
-                                   nullptr;
-    fdd.orco = (const float(*)[3])CustomData_get_layer(&mesh_eval->vert_data, CD_ORCO);
+    fdd.uv_map = (uv_idx != -1) ? (const float2 *)CustomData_get_layer_n(
+                                      &mesh_eval->corner_data, CD_PROP_FLOAT2, uv_idx) :
+                                  nullptr;
+    fdd.orco = (const float (*)[3])CustomData_get_layer(&mesh_eval->vert_data, CD_ORCO);
 
     make_child_duplis(ctx, &fdd, make_child_duplis_faces_from_mesh);
   }
@@ -1804,39 +1804,49 @@ static const DupliGenerator *get_dupli_generator(const DupliContext *ctx)
 /** \name Dupli-Container Implementation
  * \{ */
 
-ListBase *object_duplilist(Depsgraph *depsgraph,
-                           Scene *sce,
-                           Object *ob,
-                           Set<const Object *> *include_objects)
+void object_duplilist(Depsgraph *depsgraph,
+                      Scene *sce,
+                      Object *ob,
+                      Set<const Object *> *include_objects,
+                      DupliList &r_duplilist)
 {
-  ListBase *duplilist = MEM_callocN<ListBase>("duplilist");
   DupliContext ctx;
   Vector<Object *> instance_stack;
   Vector<short> dupli_gen_type_stack({0});
   instance_stack.append(ob);
-  init_context(
-      &ctx, depsgraph, sce, ob, nullptr, include_objects, instance_stack, dupli_gen_type_stack);
+  init_context(&ctx,
+               depsgraph,
+               sce,
+               ob,
+               nullptr,
+               include_objects,
+               instance_stack,
+               dupli_gen_type_stack,
+               r_duplilist);
   if (ctx.gen) {
-    ctx.duplilist = duplilist;
     ctx.gen->make_duplis(&ctx);
   }
-
-  return duplilist;
 }
 
-ListBase *object_duplilist_preview(Depsgraph *depsgraph,
-                                   Scene *sce,
-                                   Object *ob_eval,
-                                   const ViewerPath *viewer_path)
+void object_duplilist_preview(Depsgraph *depsgraph,
+                              Scene *sce,
+                              Object *ob_eval,
+                              const ViewerPath *viewer_path,
+                              DupliList &r_duplilist)
 {
-  ListBase *duplilist = MEM_callocN<ListBase>("duplilist");
   DupliContext ctx;
   Vector<Object *> instance_stack;
   Vector<short> dupli_gen_type_stack({0});
   instance_stack.append(ob_eval);
-  init_context(
-      &ctx, depsgraph, sce, ob_eval, nullptr, nullptr, instance_stack, dupli_gen_type_stack);
-  ctx.duplilist = duplilist;
+  init_context(&ctx,
+               depsgraph,
+               sce,
+               ob_eval,
+               nullptr,
+               nullptr,
+               instance_stack,
+               dupli_gen_type_stack,
+               r_duplilist);
 
   Object *ob_orig = DEG_get_original(ob_eval);
 
@@ -1851,15 +1861,16 @@ ListBase *object_duplilist_preview(Depsgraph *depsgraph,
     if (const geo_log::ViewerNodeLog *viewer_log =
             geo_log::GeoNodesLog::find_viewer_node_log_for_path(*viewer_path))
     {
-      ctx.preview_base_geometry = &viewer_log->geometry;
-      make_duplis_geometry_set_impl(&ctx,
-                                    viewer_log->geometry,
-                                    ob_eval->object_to_world().ptr(),
-                                    true,
-                                    ob_eval->type == OB_CURVES);
+      if (const blender::bke::GeometrySet *viewer_geometry = viewer_log->main_geometry()) {
+        ctx.preview_base_geometry = &*viewer_geometry;
+        make_duplis_geometry_set_impl(&ctx,
+                                      *viewer_geometry,
+                                      ob_eval->object_to_world().ptr(),
+                                      true,
+                                      ob_eval->type == OB_CURVES);
+      }
     }
   }
-  return duplilist;
 }
 
 blender::bke::Instances object_duplilist_legacy_instances(Depsgraph &depsgraph,
@@ -1868,19 +1879,25 @@ blender::bke::Instances object_duplilist_legacy_instances(Depsgraph &depsgraph,
 {
   using namespace blender;
 
-  ListBase *duplilist = MEM_callocN<ListBase>("duplilist");
   DupliContext ctx;
+  DupliList duplilist;
   Vector<Object *> instance_stack({&ob});
   Vector<short> dupli_gen_type_stack({0});
 
-  init_context(
-      &ctx, &depsgraph, &scene, &ob, nullptr, nullptr, instance_stack, dupli_gen_type_stack);
+  init_context(&ctx,
+               &depsgraph,
+               &scene,
+               &ob,
+               nullptr,
+               nullptr,
+               instance_stack,
+               dupli_gen_type_stack,
+               duplilist);
   if (ctx.gen == &gen_dupli_geometry_set) {
     /* These are not legacy instances. */
     return {};
   }
   if (ctx.gen) {
-    ctx.duplilist = duplilist;
     ctx.gen->make_duplis(&ctx);
   }
   const bool is_particle_duplis = ctx.gen == &gen_dupli_particles;
@@ -1889,12 +1906,12 @@ blender::bke::Instances object_duplilist_legacy_instances(Depsgraph &depsgraph,
   const int level_to_use = is_particle_duplis ? 1 : 0;
 
   Vector<DupliObject *> top_level_duplis;
-  LISTBASE_FOREACH (DupliObject *, dob, duplilist) {
-    BLI_assert(dob->ob != &ob);
+  for (DupliObject &dob : duplilist) {
+    BLI_assert(dob.ob != &ob);
     /* We only need the top level instances in the end, because when #Instances references an
      * object, it implicitly also references all instances of that object. */
-    if (dob->level == level_to_use) {
-      top_level_duplis.append(dob);
+    if (dob.level == level_to_use) {
+      top_level_duplis.append(&dob);
     }
   }
 
@@ -1933,16 +1950,9 @@ blender::bke::Instances object_duplilist_legacy_instances(Depsgraph &depsgraph,
   }
   instances_ids.finish();
 
-  free_object_duplilist(duplilist);
   return top_level_instances;
 }
 
-void free_object_duplilist(ListBase *lb)
-{
-  BLI_freelistN(lb);
-  MEM_freeN(lb);
-}
-
 /** \} */
 
 /* -------------------------------------------------------------------- */
@@ -1985,8 +1995,8 @@ static bool find_geonode_attribute_rgba(const DupliObject *dupli,
   return false;
 }
 
-/** Lookup an arbitrary RNA property and convert it to RGBA if possible. */
-static bool find_rna_property_rgba(PointerRNA *id_ptr, const char *name, float r_data[4])
+/** Lookup an arbitrary Custom or RNA property and convert it to RGBA if possible. */
+static bool find_property_rgba(PointerRNA *id_ptr, const char *name, float r_data[4])
 {
   if (id_ptr->data == nullptr) {
     return false;
@@ -2056,10 +2066,10 @@ static bool find_rna_property_rgba(PointerRNA *id_ptr, const char *name, float r
   return false;
 }
 
-static bool find_rna_property_rgba(const ID *id, const char *name, float r_data[4])
+static bool find_property_rgba(const ID *id, const char *name, float r_data[4])
 {
   PointerRNA ptr = RNA_id_pointer_create(const_cast<ID *>(id));
-  return find_rna_property_rgba(&ptr, name, r_data);
+  return find_property_rgba(&ptr, name, r_data);
 }
 
 bool BKE_object_dupli_find_rgba_attribute(const Object *ob,
@@ -2072,7 +2082,7 @@ bool BKE_object_dupli_find_rgba_attribute(const Object *ob,
   if (dupli && dupli->particle_system) {
     const ParticleSettings *settings = dupli->particle_system->part;
 
-    if (find_rna_property_rgba(&settings->id, name, r_value)) {
+    if (find_property_rgba(&settings->id, name, r_value)) {
       return true;
     }
   }
@@ -2083,18 +2093,18 @@ bool BKE_object_dupli_find_rgba_attribute(const Object *ob,
   }
 
   /* Check the dupli parent object. */
-  if (dupli_parent && find_rna_property_rgba(&dupli_parent->id, name, r_value)) {
+  if (dupli_parent && find_property_rgba(&dupli_parent->id, name, r_value)) {
     return true;
   }
 
   /* Check the main object. */
   if (ob) {
-    if (find_rna_property_rgba(&ob->id, name, r_value)) {
+    if (find_property_rgba(&ob->id, name, r_value)) {
       return true;
     }
 
     /* Check the main object data (e.g. mesh). */
-    if (ob->data && find_rna_property_rgba((const ID *)ob->data, name, r_value)) {
+    if (ob->data && find_property_rgba((const ID *)ob->data, name, r_value)) {
       return true;
     }
   }
@@ -2112,16 +2122,16 @@ bool BKE_view_layer_find_rgba_attribute(const Scene *scene,
     PointerRNA layer_ptr = RNA_pointer_create_discrete(
         &const_cast<ID &>(scene->id), &RNA_ViewLayer, const_cast<ViewLayer *>(layer));
 
-    if (find_rna_property_rgba(&layer_ptr, name, r_value)) {
+    if (find_property_rgba(&layer_ptr, name, r_value)) {
       return true;
     }
   }
 
-  if (find_rna_property_rgba(&scene->id, name, r_value)) {
+  if (find_property_rgba(&scene->id, name, r_value)) {
     return true;
   }
 
-  if (scene->world && find_rna_property_rgba(&scene->world->id, name, r_value)) {
+  if (scene->world && find_property_rgba(&scene->world->id, name, r_value)) {
     return true;
   }
 
