@@ -879,4 +879,22 @@ TEST(math_rotation, DualQuaternionTransform)
   }
 }
 
+TEST(math_axis_angle, AxisAngleFromQuaternion)
+{
+  {
+    const math::AxisAngle axis_angle({0.0f, 1.0f, 0.0f}, math::AngleRadian(0));
+    const math::Quaternion quaternion(1.0f, {0.0f, 0.0f, 0.0f});
+    const math::AxisAngle from_quaternion = math::to_axis_angle(quaternion);
+    EXPECT_V3_NEAR(axis_angle.axis(), from_quaternion.axis(), 1e-6);
+    EXPECT_NEAR(axis_angle.angle().radian(), from_quaternion.angle().radian(), 1e-6);
+  }
+  {
+    const math::AxisAngle axis_angle({0.0f, -1.0f, 0.0f}, math::AngleRadian(0));
+    const math::Quaternion quaternion(-1.0f, {0.0f, 0.0f, 0.0f});
+    const math::AxisAngle from_quaternion = math::to_axis_angle(quaternion);
+    EXPECT_V3_NEAR(axis_angle.axis(), from_quaternion.axis(), 1e-6);
+    EXPECT_NEAR(axis_angle.angle().radian(), from_quaternion.angle().radian(), 1e-6);
+  }
+}
+
 }  // namespace blender::math::tests
