@@ -18,6 +18,7 @@
 #include "BLI_task.hh"
 
 #include "GPU_capabilities.hh"
+#include "GPU_debug.hh"
 
 #include "draw_cache_extract.hh"
 #include "draw_subdivision.hh"
@@ -94,7 +95,9 @@ static bool use_normals_simplify(const Scene &scene, const MeshRenderData &mr)
   if (!meta_data) {
     return false;
   }
-  if (meta_data->domain == bke::AttrDomain::Corner && meta_data->data_type == CD_PROP_INT16_2D) {
+  if (meta_data->domain == bke::AttrDomain::Corner &&
+      meta_data->data_type == bke::AttrType::Int16_2D)
+  {
     return true;
   }
   return false;
@@ -186,14 +189,20 @@ void mesh_buffer_cache_create_requested(TaskGraph & /*task_graph*/,
       case IBOType::LinesAdjacency:
         created_ibos[i] = extract_lines_adjacency(mr, cache.is_manifold);
         break;
-      case IBOType::UVLines:
-        created_ibos[i] = extract_edituv_lines(mr, false);
+      case IBOType::UVTris:
+        created_ibos[i] = extract_edituv_tris(mr, false);
         break;
       case IBOType::EditUVTris:
-        created_ibos[i] = extract_edituv_tris(mr);
+        created_ibos[i] = extract_edituv_tris(mr, true);
+        break;
+      case IBOType::AllUVLines:
+        created_ibos[i] = extract_edituv_lines(mr, UvExtractionMode::All);
+        break;
+      case IBOType::UVLines:
+        created_ibos[i] = extract_edituv_lines(mr, UvExtractionMode::Selection);
         break;
       case IBOType::EditUVLines:
-        created_ibos[i] = extract_edituv_lines(mr, true);
+        created_ibos[i] = extract_edituv_lines(mr, UvExtractionMode::Edit);
         break;
       case IBOType::EditUVPoints:
         created_ibos[i] = extract_edituv_points(mr);
@@ -302,6 +311,9 @@ void mesh_buffer_cache_create_requested(TaskGraph & /*task_graph*/,
       case VBOType::VertexNormal:
         created_vbos[i] = extract_vert_normals(mr);
         break;
+      case VBOType::PaintOverlayFlag:
+        created_vbos[i] = extract_paint_overlay_flags(mr);
+        break;
     }
   });
 
@@ -353,6 +365,9 @@ void mesh_buffer_cache_create_requested_subdiv(MeshBatchCache &cache,
     return;
   }
 
+  static gpu::DebugScope subdiv_extract_scope = {"SubdivExtraction"};
+  auto capture = subdiv_extract_scope.scoped_capture();
+
   if (vbos_to_create.contains(VBOType::Position) || vbos_to_create.contains(VBOType::Orco)) {
     gpu::VertBufPtr orco_vbo;
     /* Don't use `add_new` because #VBOType::Orco might be requested after #VBOType::Position
@@ -431,11 +446,19 @@ void mesh_buffer_cache_create_requested_subdiv(MeshBatchCache &cache,
         face_dot_position_vbo,
         vbos_to_create.contains(VBOType::FaceDotNormal) ? &face_dot_normal_vbo : nullptr,
         face_dot_ibo);
-    buffers.vbos.add_new(VBOType::FaceDotPosition, std::move(face_dot_position_vbo));
+    if (vbos_to_create.contains(VBOType::FaceDotPosition)) {
+      buffers.vbos.add_new(VBOType::FaceDotPosition, std::move(face_dot_position_vbo));
+    }
     if (face_dot_normal_vbo) {
       buffers.vbos.add_new(VBOType::FaceDotNormal, std::move(face_dot_normal_vbo));
     }
-    buffers.ibos.add_new(IBOType::FaceDots, std::move(face_dot_ibo));
+    if (ibos_to_create.contains(IBOType::FaceDots)) {
+      buffers.ibos.add_new(IBOType::FaceDots, std::move(face_dot_ibo));
+    }
+  }
+  if (vbos_to_create.contains(VBOType::PaintOverlayFlag)) {
+    buffers.vbos.add_new(VBOType::PaintOverlayFlag,
+                         extract_paint_overlay_flags_subdiv(mr, subdiv_cache));
   }
   if (ibos_to_create.contains(IBOType::LinesPaintMask)) {
     buffers.ibos.add_new(IBOType::LinesPaintMask,
@@ -452,8 +475,14 @@ void mesh_buffer_cache_create_requested_subdiv(MeshBatchCache &cache,
     /* Make sure UVs are computed before edituv stuffs. */
     buffers.vbos.add_new(VBOType::UVs, extract_uv_maps_subdiv(subdiv_cache, cache));
   }
+  if (ibos_to_create.contains(IBOType::AllUVLines)) {
+    buffers.ibos.add_new(IBOType::AllUVLines,
+                         extract_edituv_lines_subdiv(mr, subdiv_cache, UvExtractionMode::All));
+  }
   if (ibos_to_create.contains(IBOType::UVLines)) {
-    buffers.ibos.add_new(IBOType::UVLines, extract_edituv_lines_subdiv(mr, subdiv_cache, false));
+    buffers.ibos.add_new(
+        IBOType::UVLines,
+        extract_edituv_lines_subdiv(mr, subdiv_cache, UvExtractionMode::Selection));
   }
   if (vbos_to_create.contains(VBOType::EditUVStretchArea)) {
     buffers.vbos.add_new(
@@ -472,7 +501,7 @@ void mesh_buffer_cache_create_requested_subdiv(MeshBatchCache &cache,
   }
   if (ibos_to_create.contains(IBOType::EditUVLines)) {
     buffers.ibos.add_new(IBOType::EditUVLines,
-                         extract_edituv_lines_subdiv(mr, subdiv_cache, true));
+                         extract_edituv_lines_subdiv(mr, subdiv_cache, UvExtractionMode::Edit));
   }
   if (ibos_to_create.contains(IBOType::EditUVPoints)) {
     buffers.ibos.add_new(IBOType::EditUVPoints, extract_edituv_points_subdiv(mr, subdiv_cache));
