@@ -13,22 +13,28 @@
 
 #include "MEM_guardedalloc.h"
 
-#include "BLI_blenlib.h"
-#include "BLI_lasso_2d.h"
+#include "BLI_function_ref.hh"
+#include "BLI_lasso_2d.hh"
+#include "BLI_listbase.h"
 #include "BLI_math_vector.h"
+#include "BLI_rect.h"
 #include "BLI_utildefines.h"
 
 #include "DNA_anim_types.h"
 #include "DNA_object_types.h"
 #include "DNA_scene_types.h"
 
-#include "BKE_fcurve.h"
-#include "BKE_nla.h"
+#include "BKE_fcurve.hh"
+#include "BKE_nla.hh"
 
 #include "ED_anim_api.hh"
 #include "ED_keyframes_edit.hh"
 #include "ED_markers.hh"
 
+#include "ANIM_action.hh"
+
+using namespace blender;
+
 /* This file defines an API and set of callback-operators for
  * non-destructive editing of keyframe data.
  *
@@ -149,24 +155,59 @@ static short agrp_keyframes_loop(KeyframeEditData *ked,
     return 0;
   }
 
-  /* only iterate over the F-Curves that are in this group */
-  LISTBASE_FOREACH (FCurve *, fcu, &agrp->channels) {
-    if (fcu->grp == agrp) {
-      if (ANIM_fcurve_keyframes_loop(ked, fcu, key_ok, key_cb, fcu_cb)) {
-        return 1;
+  /* Legacy actions. */
+  if (agrp->wrap().is_legacy()) {
+    LISTBASE_FOREACH (FCurve *, fcu, &agrp->channels) {
+      if (fcu->grp == agrp) {
+        if (ANIM_fcurve_keyframes_loop(ked, fcu, key_ok, key_cb, fcu_cb)) {
+          return 1;
+        }
       }
     }
+    return 0;
+  }
+
+  /* Layered actions. */
+  animrig::Channelbag &channelbag = agrp->channelbag->wrap();
+  Span<FCurve *> fcurves = channelbag.fcurves().slice(agrp->fcurve_range_start,
+                                                      agrp->fcurve_range_length);
+  for (FCurve *fcurve : fcurves) {
+    if (ANIM_fcurve_keyframes_loop(ked, fcurve, key_ok, key_cb, fcu_cb)) {
+      return 1;
+    }
   }
 
   return 0;
 }
 
+/* Loop over all keyframes in the layered Action. */
+static short action_layered_keyframes_loop(KeyframeEditData *ked,
+                                           animrig::Action &action,
+                                           animrig::Slot *slot,
+                                           KeyframeEditFunc key_ok,
+                                           KeyframeEditFunc key_cb,
+                                           FcuEditFunc fcu_cb)
+{
+  if (!slot) {
+    /* Valid situation, and will not have any FCurves. */
+    return 0;
+  }
+
+  Span<FCurve *> fcurves = animrig::fcurves_for_action_slot(action, slot->handle);
+  for (FCurve *fcurve : fcurves) {
+    if (ANIM_fcurve_keyframes_loop(ked, fcurve, key_ok, key_cb, fcu_cb)) {
+      return 1;
+    }
+  }
+  return 0;
+}
+
 /* This function is used to loop over the keyframe data in an Action */
-static short act_keyframes_loop(KeyframeEditData *ked,
-                                bAction *act,
-                                KeyframeEditFunc key_ok,
-                                KeyframeEditFunc key_cb,
-                                FcuEditFunc fcu_cb)
+static short action_legacy_keyframes_loop(KeyframeEditData *ked,
+                                          bAction *act,
+                                          KeyframeEditFunc key_ok,
+                                          KeyframeEditFunc key_cb,
+                                          FcuEditFunc fcu_cb)
 {
   /* sanity check */
   if (act == nullptr) {
@@ -214,6 +255,8 @@ static short ob_keyframes_loop(KeyframeEditData *ked,
   ac.ads = ads;
   ac.data = &dummy_chan;
   ac.datatype = ANIMCONT_CHANNEL;
+  ac.filters.flag = eDopeSheet_FilterFlag(ads->filterflag);
+  ac.filters.flag2 = eDopeSheet_FilterFlag2(ads->filterflag2);
 
   /* get F-Curves to take keyframes from */
   filter = ANIMFILTER_DATA_VISIBLE | ANIMFILTER_FCURVESONLY;
@@ -223,7 +266,8 @@ static short ob_keyframes_loop(KeyframeEditData *ked,
   /* Loop through each F-Curve, applying the operation as required,
    * but stopping on the first one. */
   LISTBASE_FOREACH (bAnimListElem *, ale, &anim_data) {
-    if (ANIM_fcurve_keyframes_loop(ked, (FCurve *)ale->data, key_ok, key_cb, fcu_cb)) {
+    if (ANIM_fcurve_keyframes_loop(ked, static_cast<FCurve *>(ale->data), key_ok, key_cb, fcu_cb))
+    {
       ret = 1;
       break;
     }
@@ -263,6 +307,8 @@ static short scene_keyframes_loop(KeyframeEditData *ked,
   ac.ads = ads;
   ac.data = &dummy_chan;
   ac.datatype = ANIMCONT_CHANNEL;
+  ac.filters.flag = eDopeSheet_FilterFlag(ads->filterflag);
+  ac.filters.flag2 = eDopeSheet_FilterFlag2(ads->filterflag2);
 
   /* get F-Curves to take keyframes from */
   filter = ANIMFILTER_DATA_VISIBLE | ANIMFILTER_FCURVESONLY;
@@ -272,7 +318,8 @@ static short scene_keyframes_loop(KeyframeEditData *ked,
   /* Loop through each F-Curve, applying the operation as required,
    * but stopping on the first one. */
   LISTBASE_FOREACH (bAnimListElem *, ale, &anim_data) {
-    if (ANIM_fcurve_keyframes_loop(ked, (FCurve *)ale->data, key_ok, key_cb, fcu_cb)) {
+    if (ANIM_fcurve_keyframes_loop(ked, static_cast<FCurve *>(ale->data), key_ok, key_cb, fcu_cb))
+    {
       ret = 1;
       break;
     }
@@ -309,6 +356,7 @@ static short summary_keyframes_loop(KeyframeEditData *ked,
     switch (ale->datatype) {
       case ALE_MASKLAY:
       case ALE_GPFRAME:
+      case ALE_GREASE_PENCIL_CEL:
         break;
 
       case ALE_FCURVE:
@@ -320,15 +368,11 @@ static short summary_keyframes_loop(KeyframeEditData *ked,
           float f1 = ked->f1;
           float f2 = ked->f2;
 
-          if (ked->iterflags & (KED_F1_NLA_UNMAP | KED_F2_NLA_UNMAP)) {
-            AnimData *adt = ANIM_nla_mapping_get(ac, ale);
-
-            if (ked->iterflags & KED_F1_NLA_UNMAP) {
-              ked->f1 = BKE_nla_tweakedit_remap(adt, f1, NLATIME_CONVERT_UNMAP);
-            }
-            if (ked->iterflags & KED_F2_NLA_UNMAP) {
-              ked->f2 = BKE_nla_tweakedit_remap(adt, f2, NLATIME_CONVERT_UNMAP);
-            }
+          if (ked->iterflags & KED_F1_NLA_UNMAP) {
+            ked->f1 = ANIM_nla_tweakedit_remap(ale, f1, NLATIME_CONVERT_UNMAP);
+          }
+          if (ked->iterflags & KED_F2_NLA_UNMAP) {
+            ked->f2 = ANIM_nla_tweakedit_remap(ale, f2, NLATIME_CONVERT_UNMAP);
           }
 
           /* now operate on the channel as per normal */
@@ -383,54 +427,44 @@ short ANIM_animchannel_keyframes_loop(KeyframeEditData *ked,
      * NOTE: must keep this code in sync with the drawing code and also the filtering code!
      */
     case ALE_GROUP: /* action group */
-      return agrp_keyframes_loop(ked, (bActionGroup *)ale->data, key_ok, key_cb, fcu_cb);
-    case ALE_ACT: /* action */
-      return act_keyframes_loop(ked, (bAction *)ale->key_data, key_ok, key_cb, fcu_cb);
+      return agrp_keyframes_loop(
+          ked, static_cast<bActionGroup *>(ale->data), key_ok, key_cb, fcu_cb);
+    case ALE_ACTION_LAYERED: { /* Layered Action. */
+      /* This assumes that the ALE_ACTION_LAYERED channel is shown in the dope-sheet context,
+       * underneath the data-block that owns `ale->adt`. So that means that the loop is limited to
+       * the keys that belong to that slot. */
+      animrig::Action &action = static_cast<bAction *>(ale->key_data)->wrap();
+      animrig::Slot *slot = action.slot_for_handle(ale->adt->slot_handle);
+      return action_layered_keyframes_loop(ked, action, slot, key_ok, key_cb, fcu_cb);
+    }
+    case ALE_ACTION_SLOT: {
+      animrig::Action *action = static_cast<animrig::Action *>(ale->key_data);
+      BLI_assert(action);
+      animrig::Slot *slot = static_cast<animrig::Slot *>(ale->data);
+      return action_layered_keyframes_loop(ked, *action, slot, key_ok, key_cb, fcu_cb);
+    }
 
+    case ALE_ACT: /* Legacy Action. */
+      return action_legacy_keyframes_loop(
+          ked, static_cast<bAction *>(ale->key_data), key_ok, key_cb, fcu_cb);
     case ALE_OB: /* object */
-      return ob_keyframes_loop(ked, ads, (Object *)ale->key_data, key_ok, key_cb, fcu_cb);
+      return ob_keyframes_loop(
+          ked, ads, static_cast<Object *>(ale->key_data), key_ok, key_cb, fcu_cb);
     case ALE_SCE: /* scene */
-      return scene_keyframes_loop(ked, ads, (Scene *)ale->data, key_ok, key_cb, fcu_cb);
+      return scene_keyframes_loop(
+          ked, ads, static_cast<Scene *>(ale->data), key_ok, key_cb, fcu_cb);
     case ALE_ALL: /* 'all' (DopeSheet summary) */
-      return summary_keyframes_loop(ked, (bAnimContext *)ale->data, key_ok, key_cb, fcu_cb);
-  }
+      return summary_keyframes_loop(
+          ked, static_cast<bAnimContext *>(ale->data), key_ok, key_cb, fcu_cb);
 
-  return 0;
-}
-
-short ANIM_animchanneldata_keyframes_loop(KeyframeEditData *ked,
-                                          bDopeSheet *ads,
-                                          void *data,
-                                          int keytype,
-                                          KeyframeEditFunc key_ok,
-                                          KeyframeEditFunc key_cb,
-                                          FcuEditFunc fcu_cb)
-{
-  /* sanity checks */
-  if (data == nullptr) {
-    return 0;
-  }
-
-  /* method to use depends on the type of keyframe data */
-  switch (keytype) {
-    /* direct keyframe data (these loops are exposed) */
-    case ALE_FCURVE: /* F-Curve */
-      return ANIM_fcurve_keyframes_loop(ked, static_cast<FCurve *>(data), key_ok, key_cb, fcu_cb);
-
-    /* indirect 'summaries' (these are not exposed directly)
-     * NOTE: must keep this code in sync with the drawing code and also the filtering code!
-     */
-    case ALE_GROUP: /* action group */
-      return agrp_keyframes_loop(ked, (bActionGroup *)data, key_ok, key_cb, fcu_cb);
-    case ALE_ACT: /* action */
-      return act_keyframes_loop(ked, (bAction *)data, key_ok, key_cb, fcu_cb);
-
-    case ALE_OB: /* object */
-      return ob_keyframes_loop(ked, ads, (Object *)data, key_ok, key_cb, fcu_cb);
-    case ALE_SCE: /* scene */
-      return scene_keyframes_loop(ked, ads, (Scene *)data, key_ok, key_cb, fcu_cb);
-    case ALE_ALL: /* 'all' (DopeSheet summary) */
-      return summary_keyframes_loop(ked, (bAnimContext *)data, key_ok, key_cb, fcu_cb);
+    case ALE_NONE:
+    case ALE_GPFRAME:
+    case ALE_MASKLAY:
+    case ALE_NLASTRIP:
+    case ALE_GREASE_PENCIL_CEL:
+    case ALE_GREASE_PENCIL_DATA:
+    case ALE_GREASE_PENCIL_GROUP:
+      break;
   }
 
   return 0;
@@ -486,62 +520,62 @@ void ANIM_editkeyframes_refresh(bAnimContext *ac)
 /* BezTriple Validation Callbacks */
 
 /* ------------------------ */
-/* Some macros to make this easier... */
 
-/* run the given check on the 3 handles:
- * - Check should be a macro, which takes the handle index as its single arg,
- *   which it substitutes later.
- * - Requires that a var, of type short, is named 'ok',
- *   and has been initialized to 0.
- */
-#define KEYFRAME_OK_CHECKS(check) \
-  { \
-    CHECK_TYPE(ok, short); \
-    if (check(1)) { \
-      ok |= KEYFRAME_OK_KEY; \
-    } \
-    if (ked && (ked->iterflags & KEYFRAME_ITER_INCL_HANDLES)) { \
-      /* Only act on visible items, so check handle visibility state. */ \
-      const bool handles_visible = ((ked->iterflags & KEYFRAME_ITER_HANDLES_DEFAULT_INVISIBLE) ? \
-                                        BEZT_ISSEL_ANY(bezt) : \
-                                        true); \
-      if (handles_visible) { \
-        if (check(0)) { \
-          ok |= KEYFRAME_OK_H1; \
-        } \
-        if (check(2)) { \
-          ok |= KEYFRAME_OK_H2; \
-        } \
-      } \
-    } \
-  } \
-  (void)0
+static bool handles_visible(KeyframeEditData *ked, BezTriple *bezt)
+{
+  const bool handles_shown = (ked->iterflags & KEYFRAME_ITER_HANDLES_INVISIBLE) == 0;
+  if (!handles_shown) {
+    return false;
+  }
+  const bool handles_shown_only_selected = ked->iterflags &
+                                           KEYFRAME_ITER_HANDLES_DEFAULT_INVISIBLE;
+
+  return handles_shown_only_selected ? BEZT_ISSEL_ANY(bezt) : true;
+}
+
+static short keyframe_ok_checks(
+    KeyframeEditData *ked,
+    BezTriple *bezt,
+    blender::FunctionRef<bool(KeyframeEditData *ked, BezTriple *bezt, const int index)> check)
+{
+  short ok = 0;
+  if (check(ked, bezt, 1)) {
+    ok |= KEYFRAME_OK_KEY;
+  }
+  if (ked && (ked->iterflags & KEYFRAME_ITER_INCL_HANDLES))
+  { /* Only act on visible items, so check handle visibility state. */
+    if (handles_visible(ked, bezt)) {
+      if (check(ked, bezt, 0)) {
+        ok |= KEYFRAME_OK_H1;
+      }
+      if (check(ked, bezt, 2)) {
+        ok |= KEYFRAME_OK_H2;
+      }
+    }
+  }
+  return ok;
+}
 
 /* ------------------------ */
 
 static short ok_bezier_frame(KeyframeEditData *ked, BezTriple *bezt)
 {
-  short ok = 0;
-
   /* frame is stored in f1 property (this float accuracy check may need to be dropped?) */
-#define KEY_CHECK_OK(_index) IS_EQF(bezt->vec[_index][0], ked->f1)
-  KEYFRAME_OK_CHECKS(KEY_CHECK_OK);
-#undef KEY_CHECK_OK
+  const short ok = keyframe_ok_checks(
+      ked, bezt, [](KeyframeEditData *ked, BezTriple *bezt, int index) -> bool {
+        return IS_EQF(bezt->vec[index][0], ked->f1);
+      });
 
-  /* return ok flags */
   return ok;
 }
 
 static short ok_bezier_framerange(KeyframeEditData *ked, BezTriple *bezt)
 {
-  short ok = 0;
+  const short ok = keyframe_ok_checks(
+      ked, bezt, [](KeyframeEditData *ked, BezTriple *bezt, int index) -> bool {
+        return (bezt->vec[index][0] > ked->f1) && (bezt->vec[index][0] < ked->f2);
+      });
 
-  /* frame range is stored in float properties */
-#define KEY_CHECK_OK(_index) ((bezt->vec[_index][0] > ked->f1) && (bezt->vec[_index][0] < ked->f2))
-  KEYFRAME_OK_CHECKS(KEY_CHECK_OK);
-#undef KEY_CHECK_OK
-
-  /* return ok flags */
   return ok;
 }
 
@@ -556,50 +590,54 @@ static short ok_bezier_selected(KeyframeEditData * /*ked*/, BezTriple *bezt)
   return 0;
 }
 
+static short ok_bezier_selected_key(KeyframeEditData * /*ked*/, BezTriple *bezt)
+{
+  /* This macro checks the beztriple key (f2) selection. */
+  if (BEZT_ISSEL_IDX(bezt, 1)) {
+    return KEYFRAME_OK_KEY;
+  }
+  return 0;
+}
+
 static short ok_bezier_value(KeyframeEditData *ked, BezTriple *bezt)
 {
-  short ok = 0;
-
   /* Value is stored in f1 property:
    * - This float accuracy check may need to be dropped?
    * - Should value be stored in f2 instead
    *   so that we won't have conflicts when using f1 for frames too?
    */
-#define KEY_CHECK_OK(_index) IS_EQF(bezt->vec[_index][1], ked->f1)
-  KEYFRAME_OK_CHECKS(KEY_CHECK_OK);
-#undef KEY_CHECK_OK
+  const short ok = keyframe_ok_checks(
+      ked, bezt, [](KeyframeEditData *ked, BezTriple *bezt, int index) -> bool {
+        return IS_EQF(bezt->vec[index][1], ked->f1);
+      });
 
-  /* return ok flags */
   return ok;
 }
 
 static short ok_bezier_valuerange(KeyframeEditData *ked, BezTriple *bezt)
 {
-  short ok = 0;
-
   /* value range is stored in float properties */
-#define KEY_CHECK_OK(_index) ((bezt->vec[_index][1] > ked->f1) && (bezt->vec[_index][1] < ked->f2))
-  KEYFRAME_OK_CHECKS(KEY_CHECK_OK);
-#undef KEY_CHECK_OK
+  const short ok = keyframe_ok_checks(
+      ked, bezt, [](KeyframeEditData *ked, BezTriple *bezt, int index) -> bool {
+        return (bezt->vec[index][1] > ked->f1) && (bezt->vec[index][1] < ked->f2);
+      });
 
-  /* return ok flags */
   return ok;
 }
 
 static short ok_bezier_region(KeyframeEditData *ked, BezTriple *bezt)
 {
   /* rect is stored in data property (it's of type rectf, but may not be set) */
-  if (ked->data) {
-    short ok = 0;
-
-#define KEY_CHECK_OK(_index) BLI_rctf_isect_pt_v(static_cast<rctf *>(ked->data), bezt->vec[_index])
-    KEYFRAME_OK_CHECKS(KEY_CHECK_OK);
-#undef KEY_CHECK_OK
-
-    /* return ok flags */
-    return ok;
+  if (!ked->data) {
+    return 0;
   }
-  return 0;
+
+  const short ok = keyframe_ok_checks(
+      ked, bezt, [](KeyframeEditData *ked, BezTriple *bezt, int index) -> bool {
+        return BLI_rctf_isect_pt_v(static_cast<rctf *>(ked->data), bezt->vec[index]);
+      });
+
+  return ok;
 }
 
 bool keyframe_region_lasso_test(const KeyframeEdit_LassoData *data_lasso, const float xy[2])
@@ -609,9 +647,7 @@ bool keyframe_region_lasso_test(const KeyframeEdit_LassoData *data_lasso, const
 
     BLI_rctf_transform_pt_v(data_lasso->rectf_view, data_lasso->rectf_scaled, xy_view, xy);
 
-    if (BLI_lasso_is_point_inside(
-            data_lasso->mcoords, data_lasso->mcoords_len, xy_view[0], xy_view[1], INT_MAX))
-    {
+    if (BLI_lasso_is_point_inside(data_lasso->mcoords, xy_view[0], xy_view[1], INT_MAX)) {
       return true;
     }
   }
@@ -622,18 +658,17 @@ bool keyframe_region_lasso_test(const KeyframeEdit_LassoData *data_lasso, const
 static short ok_bezier_region_lasso(KeyframeEditData *ked, BezTriple *bezt)
 {
   /* check for lasso customdata (KeyframeEdit_LassoData) */
-  if (ked->data) {
-    short ok = 0;
-
-#define KEY_CHECK_OK(_index) \
-  keyframe_region_lasso_test(static_cast<KeyframeEdit_LassoData *>(ked->data), bezt->vec[_index])
-    KEYFRAME_OK_CHECKS(KEY_CHECK_OK);
-#undef KEY_CHECK_OK
-
-    /* return ok flags */
-    return ok;
+  if (!ked->data) {
+    return 0;
   }
-  return 0;
+
+  const short ok = keyframe_ok_checks(
+      ked, bezt, [](KeyframeEditData *ked, BezTriple *bezt, int index) -> bool {
+        return keyframe_region_lasso_test(static_cast<KeyframeEdit_LassoData *>(ked->data),
+                                          bezt->vec[index]);
+      });
+
+  return ok;
 }
 
 static short ok_bezier_channel_lasso(KeyframeEditData *ked, BezTriple *bezt)
@@ -682,18 +717,17 @@ bool keyframe_region_circle_test(const KeyframeEdit_CircleData *data_circle, con
 static short ok_bezier_region_circle(KeyframeEditData *ked, BezTriple *bezt)
 {
   /* check for circle select customdata (KeyframeEdit_CircleData) */
-  if (ked->data) {
-    short ok = 0;
-
-#define KEY_CHECK_OK(_index) \
-  keyframe_region_circle_test(static_cast<KeyframeEdit_CircleData *>(ked->data), bezt->vec[_index])
-    KEYFRAME_OK_CHECKS(KEY_CHECK_OK);
-#undef KEY_CHECK_OK
-
-    /* return ok flags */
-    return ok;
+  if (!ked->data) {
+    return 0;
   }
-  return 0;
+
+  const short ok = keyframe_ok_checks(
+      ked, bezt, [](KeyframeEditData *ked, BezTriple *bezt, int index) -> bool {
+        return keyframe_region_circle_test(static_cast<KeyframeEdit_CircleData *>(ked->data),
+                                           bezt->vec[index]);
+      });
+
+  return ok;
 }
 
 static short ok_bezier_channel_circle(KeyframeEditData *ked, BezTriple *bezt)
@@ -735,8 +769,11 @@ KeyframeEditFunc ANIM_editkeyframes_ok(short mode)
       /* only if bezt falls within the specified frame range (floats) */
       return ok_bezier_framerange;
     case BEZT_OK_SELECTED:
-      /* only if bezt is selected (self) */
+      /* only if bezt is selected (any of f1, f2, f3) */
       return ok_bezier_selected;
+    case BEZT_OK_SELECTED_KEY:
+      /* only if bezt is selected (f2 is enough) */
+      return ok_bezier_selected_key;
     case BEZT_OK_VALUE:
       /* only if bezt value matches (float) */
       return ok_bezier_value;
@@ -788,10 +825,19 @@ short bezt_calc_average(KeyframeEditData *ked, BezTriple *bezt)
 short bezt_to_cfraelem(KeyframeEditData *ked, BezTriple *bezt)
 {
   /* only if selected */
-  if (bezt->f2 & SELECT) {
-    CfraElem *ce = static_cast<CfraElem *>(MEM_callocN(sizeof(CfraElem), "cfraElem"));
-    BLI_addtail(&ked->list, ce);
+  if ((bezt->f2 & SELECT) == 0) {
+    return 0;
+  }
 
+  CfraElem *ce = MEM_callocN<CfraElem>("cfraElem");
+  BLI_addtail(&ked->list, ce);
+
+  /* bAnimListElem so we can do NLA mapping, we want the cfra to be in "global" time */
+  bAnimListElem *ale = static_cast<bAnimListElem *>(ked->data);
+  if (ale != nullptr) {
+    ce->cfra = ANIM_nla_tweakedit_remap(ale, bezt->vec[1][0], NLATIME_CONVERT_MAP);
+  }
+  else {
     ce->cfra = bezt->vec[1][0];
   }
 
@@ -800,7 +846,7 @@ short bezt_to_cfraelem(KeyframeEditData *ked, BezTriple *bezt)
 
 void bezt_remap_times(KeyframeEditData *ked, BezTriple *bezt)
 {
-  KeyframeEditCD_Remap *rmap = (KeyframeEditCD_Remap *)ked->data;
+  KeyframeEditCD_Remap *rmap = static_cast<KeyframeEditCD_Remap *>(ked->data);
   const float scale = (rmap->newMax - rmap->newMin) / (rmap->oldMax - rmap->oldMin);
 
   /* perform transform on all three handles unless indicated otherwise */
@@ -818,7 +864,7 @@ void bezt_remap_times(KeyframeEditData *ked, BezTriple *bezt)
 static short snap_bezier_nearest(KeyframeEditData * /*ked*/, BezTriple *bezt)
 {
   if (bezt->f2 & SELECT) {
-    bezt->vec[1][0] = float(floorf(bezt->vec[1][0] + 0.5f));
+    BKE_fcurve_keyframe_move_time_with_handles(bezt, floorf(bezt->vec[1][0] + 0.5f));
   }
   return 0;
 }
@@ -827,10 +873,10 @@ static short snap_bezier_nearest(KeyframeEditData * /*ked*/, BezTriple *bezt)
 static short snap_bezier_nearestsec(KeyframeEditData *ked, BezTriple *bezt)
 {
   const Scene *scene = ked->scene;
-  const float secf = float(FPS);
+  const float secf = float(scene->frames_per_second());
 
   if (bezt->f2 & SELECT) {
-    bezt->vec[1][0] = float(floorf(bezt->vec[1][0] / secf + 0.5f)) * secf;
+    BKE_fcurve_keyframe_move_time_with_handles(bezt, floorf(bezt->vec[1][0] / secf + 0.5f) * secf);
   }
   return 0;
 }
@@ -840,7 +886,7 @@ static short snap_bezier_cframe(KeyframeEditData *ked, BezTriple *bezt)
 {
   const Scene *scene = ked->scene;
   if (bezt->f2 & SELECT) {
-    bezt->vec[1][0] = float(scene->r.cfra);
+    BKE_fcurve_keyframe_move_time_with_handles(bezt, float(scene->r.cfra));
   }
   return 0;
 }
@@ -849,7 +895,8 @@ static short snap_bezier_cframe(KeyframeEditData *ked, BezTriple *bezt)
 static short snap_bezier_nearmarker(KeyframeEditData *ked, BezTriple *bezt)
 {
   if (bezt->f2 & SELECT) {
-    bezt->vec[1][0] = float(ED_markers_find_nearest_marker_time(&ked->list, bezt->vec[1][0]));
+    BKE_fcurve_keyframe_move_time_with_handles(
+        bezt, float(ED_markers_find_nearest_marker_time(&ked->list, bezt->vec[1][0])));
   }
   return 0;
 }
@@ -874,7 +921,7 @@ static short snap_bezier_horizontal(KeyframeEditData * /*ked*/, BezTriple *bezt)
 static short snap_bezier_time(KeyframeEditData *ked, BezTriple *bezt)
 {
   if (bezt->f2 & SELECT) {
-    bezt->vec[1][0] = ked->f1;
+    BKE_fcurve_keyframe_move_time_with_handles(bezt, ked->f1);
   }
   return 0;
 }
@@ -883,7 +930,7 @@ static short snap_bezier_time(KeyframeEditData *ked, BezTriple *bezt)
 static short snap_bezier_value(KeyframeEditData *ked, BezTriple *bezt)
 {
   if (bezt->f2 & SELECT) {
-    bezt->vec[1][1] = ked->f1;
+    BKE_fcurve_keyframe_move_value_with_handles(bezt, ked->f1);
   }
   return 0;
 }
@@ -921,8 +968,8 @@ static void mirror_bezier_xaxis_ex(BezTriple *bezt, const float center)
   }
   swap_v3_v3(bezt->vec[0], bezt->vec[2]);
 
-  SWAP(uint8_t, bezt->h1, bezt->h2);
-  SWAP(uint8_t, bezt->f1, bezt->f3);
+  std::swap(bezt->h1, bezt->h2);
+  std::swap(bezt->f1, bezt->f3);
 }
 
 static void mirror_bezier_yaxis_ex(BezTriple *bezt, const float center)
@@ -1388,7 +1435,7 @@ KeyframeEditFunc ANIM_editkeyframes_ipo(short mode)
 static short set_keytype_keyframe(KeyframeEditData * /*ked*/, BezTriple *bezt)
 {
   if (bezt->f2 & SELECT) {
-    BEZKEYTYPE(bezt) = BEZT_KEYTYPE_KEYFRAME;
+    BEZKEYTYPE_LVALUE(bezt) = BEZT_KEYTYPE_KEYFRAME;
   }
   return 0;
 }
@@ -1396,7 +1443,7 @@ static short set_keytype_keyframe(KeyframeEditData * /*ked*/, BezTriple *bezt)
 static short set_keytype_breakdown(KeyframeEditData * /*ked*/, BezTriple *bezt)
 {
   if (bezt->f2 & SELECT) {
-    BEZKEYTYPE(bezt) = BEZT_KEYTYPE_BREAKDOWN;
+    BEZKEYTYPE_LVALUE(bezt) = BEZT_KEYTYPE_BREAKDOWN;
   }
   return 0;
 }
@@ -1404,7 +1451,7 @@ static short set_keytype_breakdown(KeyframeEditData * /*ked*/, BezTriple *bezt)
 static short set_keytype_extreme(KeyframeEditData * /*ked*/, BezTriple *bezt)
 {
   if (bezt->f2 & SELECT) {
-    BEZKEYTYPE(bezt) = BEZT_KEYTYPE_EXTREME;
+    BEZKEYTYPE_LVALUE(bezt) = BEZT_KEYTYPE_EXTREME;
   }
   return 0;
 }
@@ -1412,7 +1459,7 @@ static short set_keytype_extreme(KeyframeEditData * /*ked*/, BezTriple *bezt)
 static short set_keytype_jitter(KeyframeEditData * /*ked*/, BezTriple *bezt)
 {
   if (bezt->f2 & SELECT) {
-    BEZKEYTYPE(bezt) = BEZT_KEYTYPE_JITTER;
+    BEZKEYTYPE_LVALUE(bezt) = BEZT_KEYTYPE_JITTER;
   }
   return 0;
 }
@@ -1420,30 +1467,43 @@ static short set_keytype_jitter(KeyframeEditData * /*ked*/, BezTriple *bezt)
 static short set_keytype_moving_hold(KeyframeEditData * /*ked*/, BezTriple *bezt)
 {
   if (bezt->f2 & SELECT) {
-    BEZKEYTYPE(bezt) = BEZT_KEYTYPE_MOVEHOLD;
+    BEZKEYTYPE_LVALUE(bezt) = BEZT_KEYTYPE_MOVEHOLD;
   }
   return 0;
 }
 
-KeyframeEditFunc ANIM_editkeyframes_keytype(short mode)
+static short set_keytype_generated(KeyframeEditData * /*ked*/, BezTriple *bezt)
 {
-  switch (mode) {
-    case BEZT_KEYTYPE_BREAKDOWN: /* breakdown */
+  if (bezt->f2 & SELECT) {
+    BEZKEYTYPE_LVALUE(bezt) = BEZT_KEYTYPE_GENERATED;
+  }
+  return 0;
+}
+
+KeyframeEditFunc ANIM_editkeyframes_keytype(const eBezTriple_KeyframeType keyframe_type)
+{
+  switch (keyframe_type) {
+    case BEZT_KEYTYPE_BREAKDOWN:
       return set_keytype_breakdown;
 
-    case BEZT_KEYTYPE_EXTREME: /* extreme keyframe */
+    case BEZT_KEYTYPE_EXTREME:
       return set_keytype_extreme;
 
-    case BEZT_KEYTYPE_JITTER: /* jitter keyframe */
+    case BEZT_KEYTYPE_JITTER:
       return set_keytype_jitter;
 
-    case BEZT_KEYTYPE_MOVEHOLD: /* moving hold */
+    case BEZT_KEYTYPE_MOVEHOLD:
       return set_keytype_moving_hold;
 
-    case BEZT_KEYTYPE_KEYFRAME: /* proper keyframe */
-    default:
+    case BEZT_KEYTYPE_KEYFRAME:
       return set_keytype_keyframe;
+
+    case BEZT_KEYTYPE_GENERATED:
+      return set_keytype_generated;
   }
+
+  BLI_assert_unreachable();
+  return nullptr;
 }
 
 /* ------- */
@@ -1503,12 +1563,8 @@ KeyframeEditFunc ANIM_editkeyframes_easing(short mode)
 static short select_bezier_add(KeyframeEditData *ked, BezTriple *bezt)
 {
   /* Only act on visible items, so check handle visibility state. */
-  const bool handles_visible = ked && ((ked->iterflags & KEYFRAME_ITER_HANDLES_DEFAULT_INVISIBLE) ?
-                                           BEZT_ISSEL_ANY(bezt) :
-                                           true);
-
   /* if we've got info on what to select, use it, otherwise select all */
-  if ((ked) && (ked->iterflags & KEYFRAME_ITER_INCL_HANDLES) && handles_visible) {
+  if ((ked) && (ked->iterflags & KEYFRAME_ITER_INCL_HANDLES) && handles_visible(ked, bezt)) {
     if (ked->curflags & KEYFRAME_OK_KEY) {
       bezt->f2 |= SELECT;
     }
@@ -1529,12 +1585,8 @@ static short select_bezier_add(KeyframeEditData *ked, BezTriple *bezt)
 static short select_bezier_subtract(KeyframeEditData *ked, BezTriple *bezt)
 {
   /* Only act on visible items, so check handle visibility state. */
-  const bool handles_visible = ked && ((ked->iterflags & KEYFRAME_ITER_HANDLES_DEFAULT_INVISIBLE) ?
-                                           BEZT_ISSEL_ANY(bezt) :
-                                           true);
-
   /* if we've got info on what to deselect, use it, otherwise deselect all */
-  if ((ked) && (ked->iterflags & KEYFRAME_ITER_INCL_HANDLES) && handles_visible) {
+  if ((ked) && (ked->iterflags & KEYFRAME_ITER_INCL_HANDLES) && handles_visible(ked, bezt)) {
     if (ked->curflags & KEYFRAME_OK_KEY) {
       bezt->f2 &= ~SELECT;
     }
@@ -1567,7 +1619,7 @@ static short select_bezier_invert(KeyframeEditData * /*ked*/, BezTriple *bezt)
   return 0;
 }
 
-KeyframeEditFunc ANIM_editkeyframes_select(short selectmode)
+KeyframeEditFunc ANIM_editkeyframes_select(const eEditKeyframes_Select selectmode)
 {
   switch (selectmode) {
     case SELECT_ADD: /* add */
@@ -1594,7 +1646,7 @@ KeyframeEditFunc ANIM_editkeyframes_select(short selectmode)
 
 static short selmap_build_bezier_more(KeyframeEditData *ked, BezTriple *bezt)
 {
-  FCurve *fcu = ked->fcu;
+  const FCurve *fcu = ked->fcu;
   char *map = static_cast<char *>(ked->data);
   int i = ked->curIndex;
 
@@ -1629,7 +1681,7 @@ static short selmap_build_bezier_more(KeyframeEditData *ked, BezTriple *bezt)
 
 static short selmap_build_bezier_less(KeyframeEditData *ked, BezTriple *bezt)
 {
-  FCurve *fcu = ked->fcu;
+  const FCurve *fcu = ked->fcu;
   char *map = static_cast<char *>(ked->data);
   int i = ked->curIndex;
 
