@@ -51,6 +51,7 @@
 #include "UI_interface.hh"
 #include "WM_api.hh"
 
+#include "ANIM_action.hh"
 #include "ANIM_armature.hh"
 #include "ANIM_bone_collections.hh"
 
@@ -86,10 +87,6 @@ const char *screen_context_dir[] = {
     "image_paint_object",
     "particle_edit_object",
     "pose_object",
-    "active_sequence_strip",       /* DEPRECATED - use "active_strip" */
-    "sequences",                   /* DEPRECATED - use "strips" */
-    "selected_sequences",          /* DEPRECATED - use "selected_strips" */
-    "selected_editable_sequences", /* DEPRECATED - use "selected_editable_strips" */
     "active_nla_track",
     "active_nla_strip",
     "selected_nla_strips", /* nla editor */
@@ -117,6 +114,7 @@ const char *screen_context_dir[] = {
     "strips",
     "selected_strips",
     "selected_editable_strips",
+    "sequencer_scene",
     nullptr,
 };
 
@@ -144,7 +142,7 @@ static eContextResult screen_ctx_visible_objects(const bContext *C, bContextData
       CTX_data_id_list_add(result, &base->object->id);
     }
   }
-  CTX_data_type_set(result, CTX_DATA_TYPE_COLLECTION);
+  CTX_data_type_set(result, ContextDataType::Collection);
   return CTX_RESULT_OK;
 }
 static eContextResult screen_ctx_selectable_objects(const bContext *C, bContextDataResult *result)
@@ -160,7 +158,7 @@ static eContextResult screen_ctx_selectable_objects(const bContext *C, bContextD
       CTX_data_id_list_add(result, &base->object->id);
     }
   }
-  CTX_data_type_set(result, CTX_DATA_TYPE_COLLECTION);
+  CTX_data_type_set(result, ContextDataType::Collection);
   return CTX_RESULT_OK;
 }
 static eContextResult screen_ctx_selected_objects(const bContext *C, bContextDataResult *result)
@@ -176,7 +174,7 @@ static eContextResult screen_ctx_selected_objects(const bContext *C, bContextDat
       CTX_data_id_list_add(result, &base->object->id);
     }
   }
-  CTX_data_type_set(result, CTX_DATA_TYPE_COLLECTION);
+  CTX_data_type_set(result, ContextDataType::Collection);
   return CTX_RESULT_OK;
 }
 static eContextResult screen_ctx_selected_editable_objects(const bContext *C,
@@ -193,7 +191,7 @@ static eContextResult screen_ctx_selected_editable_objects(const bContext *C,
       CTX_data_id_list_add(result, &base->object->id);
     }
   }
-  CTX_data_type_set(result, CTX_DATA_TYPE_COLLECTION);
+  CTX_data_type_set(result, ContextDataType::Collection);
   return CTX_RESULT_OK;
 }
 static eContextResult screen_ctx_editable_objects(const bContext *C, bContextDataResult *result)
@@ -210,7 +208,7 @@ static eContextResult screen_ctx_editable_objects(const bContext *C, bContextDat
       CTX_data_id_list_add(result, &base->object->id);
     }
   }
-  CTX_data_type_set(result, CTX_DATA_TYPE_COLLECTION);
+  CTX_data_type_set(result, ContextDataType::Collection);
   return CTX_RESULT_OK;
 }
 static eContextResult screen_ctx_objects_in_mode(const bContext *C, bContextDataResult *result)
@@ -228,7 +226,7 @@ static eContextResult screen_ctx_objects_in_mode(const bContext *C, bContextData
     }
     FOREACH_OBJECT_IN_MODE_END;
   }
-  CTX_data_type_set(result, CTX_DATA_TYPE_COLLECTION);
+  CTX_data_type_set(result, ContextDataType::Collection);
   return CTX_RESULT_OK;
 }
 static eContextResult screen_ctx_objects_in_mode_unique_data(const bContext *C,
@@ -254,7 +252,7 @@ static eContextResult screen_ctx_objects_in_mode_unique_data(const bContext *C,
     }
     FOREACH_OBJECT_IN_MODE_END;
   }
-  CTX_data_type_set(result, CTX_DATA_TYPE_COLLECTION);
+  CTX_data_type_set(result, ContextDataType::Collection);
   return CTX_RESULT_OK;
 }
 static eContextResult screen_ctx_visible_or_editable_bones_(const bContext *C,
@@ -280,7 +278,7 @@ static eContextResult screen_ctx_visible_or_editable_bones_(const bContext *C,
       /* Attention: X-Axis Mirroring is also handled here... */
       LISTBASE_FOREACH (EditBone *, ebone, arm->edbo) {
         /* first and foremost, bone must be visible and selected */
-        if (blender::animrig::bone_is_visible_editbone(arm, ebone)) {
+        if (blender::animrig::bone_is_visible(arm, ebone)) {
           /* Get 'x-axis mirror equivalent' bone if the X-Axis Mirroring option is enabled
            * so that most users of this data don't need to explicitly check for it themselves.
            *
@@ -307,7 +305,7 @@ static eContextResult screen_ctx_visible_or_editable_bones_(const bContext *C,
             /* only include bones if visible */
             CTX_data_list_add(result, &arm->id, &RNA_EditBone, ebone);
 
-            if ((flipbone) && blender::animrig::bone_is_visible_editbone(arm, flipbone) == 0) {
+            if ((flipbone) && blender::animrig::bone_is_visible(arm, flipbone) == 0) {
               CTX_data_list_add(result, &arm->id, &RNA_EditBone, flipbone);
             }
           }
@@ -315,7 +313,7 @@ static eContextResult screen_ctx_visible_or_editable_bones_(const bContext *C,
       }
     }
 
-    CTX_data_type_set(result, CTX_DATA_TYPE_COLLECTION);
+    CTX_data_type_set(result, ContextDataType::Collection);
     return CTX_RESULT_OK;
   }
   return CTX_RESULT_NO_DATA;
@@ -350,9 +348,7 @@ static eContextResult screen_ctx_selected_bones_(const bContext *C,
       /* Attention: X-Axis Mirroring is also handled here... */
       LISTBASE_FOREACH (EditBone *, ebone, arm->edbo) {
         /* first and foremost, bone must be visible and selected */
-        if (blender::animrig::bone_is_visible_editbone(arm, ebone) &&
-            (ebone->flag & BONE_SELECTED))
-        {
+        if (blender::animrig::bone_is_visible(arm, ebone) && (ebone->flag & BONE_SELECTED)) {
           /* Get 'x-axis mirror equivalent' bone if the X-Axis Mirroring option is enabled
            * so that most users of this data don't need to explicitly check for it themselves.
            *
@@ -387,7 +383,7 @@ static eContextResult screen_ctx_selected_bones_(const bContext *C,
       }
     }
 
-    CTX_data_type_set(result, CTX_DATA_TYPE_COLLECTION);
+    CTX_data_type_set(result, ContextDataType::Collection);
     return CTX_RESULT_OK;
   }
   return CTX_RESULT_NO_DATA;
@@ -426,7 +422,7 @@ static eContextResult screen_ctx_visible_pose_bones(const bContext *C, bContextD
       }
       FOREACH_OBJECT_IN_MODE_END;
     }
-    CTX_data_type_set(result, CTX_DATA_TYPE_COLLECTION);
+    CTX_data_type_set(result, ContextDataType::Collection);
     return CTX_RESULT_OK;
   }
   return CTX_RESULT_NO_DATA;
@@ -455,7 +451,7 @@ static eContextResult screen_ctx_selected_pose_bones(const bContext *C, bContext
       }
       FOREACH_OBJECT_IN_MODE_END;
     }
-    CTX_data_type_set(result, CTX_DATA_TYPE_COLLECTION);
+    CTX_data_type_set(result, ContextDataType::Collection);
     return CTX_RESULT_OK;
   }
   return CTX_RESULT_NO_DATA;
@@ -482,7 +478,7 @@ static eContextResult screen_ctx_selected_pose_bones_from_active_object(const bC
       }
       FOREACH_PCHAN_SELECTED_IN_OBJECT_END;
     }
-    CTX_data_type_set(result, CTX_DATA_TYPE_COLLECTION);
+    CTX_data_type_set(result, ContextDataType::Collection);
     return CTX_RESULT_OK;
   }
   return CTX_RESULT_NO_DATA;
@@ -556,7 +552,7 @@ static eContextResult screen_ctx_property(const bContext *C, bContextDataResult
       index = -1;
     }
 
-    CTX_data_type_set(result, CTX_DATA_TYPE_PROPERTY);
+    CTX_data_type_set(result, ContextDataType::Property);
     CTX_data_pointer_set_ptr(result, &ptr);
     CTX_data_prop_set(result, prop, index);
   }
@@ -672,67 +668,6 @@ static eContextResult screen_ctx_pose_object(const bContext *C, bContextDataResu
   }
   return CTX_RESULT_OK;
 }
-static eContextResult screen_ctx_active_sequence_strip(const bContext *C,
-                                                       bContextDataResult *result)
-{
-  wmWindow *win = CTX_wm_window(C);
-  Scene *scene = WM_window_get_active_scene(win);
-  Strip *strip = blender::seq::select_active_get(scene);
-  if (strip) {
-    CTX_data_pointer_set(result, &scene->id, &RNA_Strip, strip);
-    return CTX_RESULT_OK;
-  }
-  return CTX_RESULT_NO_DATA;
-}
-static eContextResult screen_ctx_sequences(const bContext *C, bContextDataResult *result)
-{
-  wmWindow *win = CTX_wm_window(C);
-  Scene *scene = WM_window_get_active_scene(win);
-  Editing *ed = blender::seq::editing_get(scene);
-  if (ed) {
-    LISTBASE_FOREACH (Strip *, strip, ed->seqbasep) {
-      CTX_data_list_add(result, &scene->id, &RNA_Strip, strip);
-    }
-    CTX_data_type_set(result, CTX_DATA_TYPE_COLLECTION);
-    return CTX_RESULT_OK;
-  }
-  return CTX_RESULT_NO_DATA;
-}
-static eContextResult screen_ctx_selected_sequences(const bContext *C, bContextDataResult *result)
-{
-  wmWindow *win = CTX_wm_window(C);
-  Scene *scene = WM_window_get_active_scene(win);
-  Editing *ed = blender::seq::editing_get(scene);
-  if (ed) {
-    LISTBASE_FOREACH (Strip *, strip, ed->seqbasep) {
-      if (strip->flag & SELECT) {
-        CTX_data_list_add(result, &scene->id, &RNA_Strip, strip);
-      }
-    }
-    CTX_data_type_set(result, CTX_DATA_TYPE_COLLECTION);
-    return CTX_RESULT_OK;
-  }
-  return CTX_RESULT_NO_DATA;
-}
-static eContextResult screen_ctx_selected_editable_sequences(const bContext *C,
-                                                             bContextDataResult *result)
-{
-  wmWindow *win = CTX_wm_window(C);
-  Scene *scene = WM_window_get_active_scene(win);
-  Editing *ed = blender::seq::editing_get(scene);
-  if (ed == nullptr) {
-    return CTX_RESULT_NO_DATA;
-  }
-
-  ListBase *channels = blender::seq::channels_displayed_get(ed);
-  LISTBASE_FOREACH (Strip *, strip, ed->seqbasep) {
-    if (strip->flag & SELECT && !blender::seq::transform_is_locked(channels, strip)) {
-      CTX_data_list_add(result, &scene->id, &RNA_Strip, strip);
-    }
-  }
-  CTX_data_type_set(result, CTX_DATA_TYPE_COLLECTION);
-  return CTX_RESULT_OK;
-}
 
 static eContextResult screen_ctx_active_nla_track(const bContext *C, bContextDataResult *result)
 {
@@ -773,7 +708,7 @@ static eContextResult screen_ctx_selected_nla_strips(const bContext *C, bContext
     }
     ANIM_animdata_freelist(&anim_data);
 
-    CTX_data_type_set(result, CTX_DATA_TYPE_COLLECTION);
+    CTX_data_type_set(result, ContextDataType::Collection);
     return CTX_RESULT_OK;
   }
   return CTX_RESULT_NO_DATA;
@@ -798,7 +733,7 @@ static eContextResult screen_ctx_selected_movieclip_tracks(const bContext *C,
     CTX_data_list_add(result, &clip->id, &RNA_MovieTrackingTrack, track);
   }
 
-  CTX_data_type_set(result, CTX_DATA_TYPE_COLLECTION);
+  CTX_data_type_set(result, ContextDataType::Collection);
   return CTX_RESULT_OK;
 }
 
@@ -847,7 +782,7 @@ static eContextResult screen_ctx_active_annotation_layer(const bContext *C,
     bGPDlayer *gpl = BKE_gpencil_layer_active_get(gpd);
 
     if (gpl) {
-      CTX_data_pointer_set(result, &gpd->id, &RNA_GPencilLayer, gpl);
+      CTX_data_pointer_set(result, &gpd->id, &RNA_AnnotationLayer, gpl);
       return CTX_RESULT_OK;
     }
   }
@@ -906,15 +841,17 @@ static eContextResult screen_ctx_sel_actions_impl(const bContext *C,
     SpaceAction *saction = (SpaceAction *)ac.sl;
 
     if (ELEM(saction->mode, SACTCONT_ACTION, SACTCONT_SHAPEKEY)) {
+      ID *active_action_id = ac.active_action ? &ac.active_action->id : nullptr;
+
       if (active_only) {
-        CTX_data_id_pointer_set(result, (ID *)saction->action);
+        CTX_data_id_pointer_set(result, active_action_id);
       }
       else {
-        if (saction->action && !(editable && !ID_IS_EDITABLE(saction->action))) {
-          CTX_data_id_list_add(result, &saction->action->id);
+        if (active_action_id && !(editable && !ID_IS_EDITABLE(active_action_id))) {
+          CTX_data_id_list_add(result, active_action_id);
         }
 
-        CTX_data_type_set(result, CTX_DATA_TYPE_COLLECTION);
+        CTX_data_type_set(result, ContextDataType::Collection);
       }
 
       return CTX_RESULT_OK;
@@ -976,7 +913,7 @@ static eContextResult screen_ctx_sel_actions_impl(const bContext *C,
   ANIM_animdata_freelist(&anim_data);
 
   if (!active_only) {
-    CTX_data_type_set(result, CTX_DATA_TYPE_COLLECTION);
+    CTX_data_type_set(result, ContextDataType::Collection);
   }
 
   return CTX_RESULT_OK;
@@ -1020,7 +957,7 @@ static eContextResult screen_ctx_sel_edit_fcurves_(const bContext *C,
 
     ANIM_animdata_freelist(&anim_data);
 
-    CTX_data_type_set(result, CTX_DATA_TYPE_COLLECTION);
+    CTX_data_type_set(result, ContextDataType::Collection);
     return CTX_RESULT_OK;
   }
   return CTX_RESULT_NO_DATA;
@@ -1110,7 +1047,7 @@ static eContextResult screen_ctx_selected_editable_keyframes(const bContext *C,
 
     ANIM_animdata_freelist(&anim_data);
 
-    CTX_data_type_set(result, CTX_DATA_TYPE_COLLECTION);
+    CTX_data_type_set(result, ContextDataType::Collection);
     return CTX_RESULT_OK;
   }
   return CTX_RESULT_NO_DATA;
@@ -1140,8 +1077,10 @@ static eContextResult screen_ctx_ui_list(const bContext *C, bContextDataResult *
 
 static eContextResult screen_ctx_active_strip(const bContext *C, bContextDataResult *result)
 {
-  wmWindow *win = CTX_wm_window(C);
-  Scene *scene = WM_window_get_active_scene(win);
+  Scene *scene = CTX_data_sequencer_scene(C);
+  if (!scene) {
+    return CTX_RESULT_NO_DATA;
+  }
   Strip *strip = blender::seq::select_active_get(scene);
   if (strip) {
     CTX_data_pointer_set(result, &scene->id, &RNA_Strip, strip);
@@ -1151,30 +1090,34 @@ static eContextResult screen_ctx_active_strip(const bContext *C, bContextDataRes
 }
 static eContextResult screen_ctx_strips(const bContext *C, bContextDataResult *result)
 {
-  wmWindow *win = CTX_wm_window(C);
-  Scene *scene = WM_window_get_active_scene(win);
+  Scene *scene = CTX_data_sequencer_scene(C);
+  if (!scene) {
+    return CTX_RESULT_NO_DATA;
+  }
   Editing *ed = blender::seq::editing_get(scene);
   if (ed) {
-    LISTBASE_FOREACH (Strip *, strip, ed->seqbasep) {
+    LISTBASE_FOREACH (Strip *, strip, ed->current_strips()) {
       CTX_data_list_add(result, &scene->id, &RNA_Strip, strip);
     }
-    CTX_data_type_set(result, CTX_DATA_TYPE_COLLECTION);
+    CTX_data_type_set(result, ContextDataType::Collection);
     return CTX_RESULT_OK;
   }
   return CTX_RESULT_NO_DATA;
 }
 static eContextResult screen_ctx_selected_strips(const bContext *C, bContextDataResult *result)
 {
-  wmWindow *win = CTX_wm_window(C);
-  Scene *scene = WM_window_get_active_scene(win);
+  Scene *scene = CTX_data_sequencer_scene(C);
+  if (!scene) {
+    return CTX_RESULT_NO_DATA;
+  }
   Editing *ed = blender::seq::editing_get(scene);
   if (ed) {
-    LISTBASE_FOREACH (Strip *, strip, ed->seqbasep) {
+    LISTBASE_FOREACH (Strip *, strip, ed->current_strips()) {
       if (strip->flag & SELECT) {
         CTX_data_list_add(result, &scene->id, &RNA_Strip, strip);
       }
     }
-    CTX_data_type_set(result, CTX_DATA_TYPE_COLLECTION);
+    CTX_data_type_set(result, ContextDataType::Collection);
     return CTX_RESULT_OK;
   }
   return CTX_RESULT_NO_DATA;
@@ -1182,22 +1125,33 @@ static eContextResult screen_ctx_selected_strips(const bContext *C, bContextData
 static eContextResult screen_ctx_selected_editable_strips(const bContext *C,
                                                           bContextDataResult *result)
 {
-  wmWindow *win = CTX_wm_window(C);
-  Scene *scene = WM_window_get_active_scene(win);
+  Scene *scene = CTX_data_sequencer_scene(C);
+  if (!scene) {
+    return CTX_RESULT_NO_DATA;
+  }
   Editing *ed = blender::seq::editing_get(scene);
   if (ed == nullptr) {
     return CTX_RESULT_NO_DATA;
   }
 
   ListBase *channels = blender::seq::channels_displayed_get(ed);
-  LISTBASE_FOREACH (Strip *, strip, ed->seqbasep) {
+  LISTBASE_FOREACH (Strip *, strip, ed->current_strips()) {
     if (strip->flag & SELECT && !blender::seq::transform_is_locked(channels, strip)) {
       CTX_data_list_add(result, &scene->id, &RNA_Strip, strip);
     }
   }
-  CTX_data_type_set(result, CTX_DATA_TYPE_COLLECTION);
+  CTX_data_type_set(result, ContextDataType::Collection);
   return CTX_RESULT_OK;
 }
+static eContextResult screen_ctx_sequencer_scene(const bContext *C, bContextDataResult *result)
+{
+  Scene *scene = CTX_data_sequencer_scene(C);
+  if (scene) {
+    CTX_data_id_pointer_set(result, &scene->id);
+    return CTX_RESULT_OK;
+  }
+  return CTX_RESULT_NO_DATA;
+}
 
 /* Registry of context callback functions. */
 
@@ -1235,11 +1189,6 @@ ensure_ed_screen_context_functions()
     map.add("image_paint_object", screen_ctx_image_paint_object);
     map.add("particle_edit_object", screen_ctx_particle_edit_object);
     map.add("pose_object", screen_ctx_pose_object);
-    map.add("active_sequence_strip", screen_ctx_active_sequence_strip); /* DEPRECATED */
-    map.add("sequences", screen_ctx_sequences);                         /* DEPRECATED */
-    map.add("selected_sequences", screen_ctx_selected_sequences);       /* DEPRECATED */
-    map.add("selected_editable_sequences",
-            screen_ctx_selected_editable_sequences); /* DEPRECATED */
     map.add("active_nla_track", screen_ctx_active_nla_track);
     map.add("active_nla_strip", screen_ctx_active_nla_strip);
     map.add("selected_nla_strips", screen_ctx_selected_nla_strips);
@@ -1265,6 +1214,7 @@ ensure_ed_screen_context_functions()
     map.add("strips", screen_ctx_strips);
     map.add("selected_strips", screen_ctx_selected_strips);
     map.add("selected_editable_strips", screen_ctx_selected_editable_strips);
+    map.add("sequencer_scene", screen_ctx_sequencer_scene);
     return map;
   }();
   return screen_context_functions;
