@@ -12,14 +12,17 @@
 #include "DNA_collection_types.h"
 #include "DNA_scene_types.h"
 
+#include "DNA_screen_types.h"
 #include "MEM_guardedalloc.h"
 
-#include "BLI_blenlib.h"
+#include "BLI_listbase.h"
+#include "BLI_math_base.h"
+#include "BLI_string_utf8.h"
 #include "BLI_utildefines.h"
 
-#include "BKE_context.h"
-#include "BKE_lib_query.h"
-#include "BKE_lib_remap.h"
+#include "BKE_context.hh"
+#include "BKE_lib_query.hh"
+#include "BKE_lib_remap.hh"
 #include "BKE_screen.hh"
 
 #include "ED_anim_api.hh"
@@ -49,25 +52,33 @@ static SpaceLink *nla_create(const ScrArea *area, const Scene *scene)
   ARegion *region;
   SpaceNla *snla;
 
-  snla = MEM_cnew<SpaceNla>("initnla");
+  snla = MEM_callocN<SpaceNla>("initnla");
   snla->spacetype = SPACE_NLA;
 
   /* allocate DopeSheet data for NLA Editor */
-  snla->ads = MEM_cnew<bDopeSheet>("NlaEdit DopeSheet");
+  snla->ads = MEM_callocN<bDopeSheet>("NlaEdit DopeSheet");
   snla->ads->source = (ID *)(scene);
 
   /* set auto-snapping settings */
   snla->flag = SNLA_SHOW_MARKERS;
 
   /* header */
-  region = MEM_cnew<ARegion>("header for nla");
+  region = BKE_area_region_new();
 
   BLI_addtail(&snla->regionbase, region);
   region->regiontype = RGN_TYPE_HEADER;
   region->alignment = (U.uiflag & USER_HEADER_BOTTOM) ? RGN_ALIGN_BOTTOM : RGN_ALIGN_TOP;
 
-  /* channel list region */
-  region = MEM_cnew<ARegion>("channel list for nla");
+  /* footer */
+  region = BKE_area_region_new();
+
+  BLI_addtail(&snla->regionbase, region);
+  region->regiontype = RGN_TYPE_FOOTER;
+  region->alignment = (U.uiflag & USER_HEADER_BOTTOM) ? RGN_ALIGN_TOP : RGN_ALIGN_BOTTOM;
+  region->flag = RGN_FLAG_HIDDEN;
+
+  /* track list region */
+  region = BKE_area_region_new();
   BLI_addtail(&snla->regionbase, region);
   region->regiontype = RGN_TYPE_CHANNELS;
   region->alignment = RGN_ALIGN_LEFT;
@@ -77,14 +88,14 @@ static SpaceLink *nla_create(const ScrArea *area, const Scene *scene)
   region->v2d.flag = V2D_VIEWSYNC_AREA_VERTICAL;
 
   /* ui buttons */
-  region = MEM_cnew<ARegion>("buttons region for nla");
+  region = BKE_area_region_new();
 
   BLI_addtail(&snla->regionbase, region);
   region->regiontype = RGN_TYPE_UI;
   region->alignment = RGN_ALIGN_RIGHT;
 
   /* main region */
-  region = MEM_cnew<ARegion>("main region for nla");
+  region = BKE_area_region_new();
 
   BLI_addtail(&snla->regionbase, region);
   region->regiontype = RGN_TYPE_WINDOW;
@@ -130,9 +141,9 @@ static void nla_init(wmWindowManager *wm, ScrArea *area)
 {
   SpaceNla *snla = static_cast<SpaceNla *>(area->spacedata.first);
 
-  /* init dopesheet data if non-existent (i.e. for old files) */
+  /* init dope-sheet data if non-existent (i.e. for old files). */
   if (snla->ads == nullptr) {
-    snla->ads = MEM_cnew<bDopeSheet>("NlaEdit DopeSheet");
+    snla->ads = MEM_callocN<bDopeSheet>("NlaEdit DopeSheet");
     wmWindow *win = WM_window_find_by_area(wm, area);
     snla->ads->source = win ? reinterpret_cast<ID *>(WM_window_get_active_scene(win)) : nullptr;
   }
@@ -151,7 +162,7 @@ static SpaceLink *nla_duplicate(SpaceLink *sl)
 }
 
 /* add handlers, stuff you only do once or on area/region changes */
-static void nla_channel_region_init(wmWindowManager *wm, ARegion *region)
+static void nla_track_region_init(wmWindowManager *wm, ARegion *region)
 {
   wmKeyMap *keymap;
 
@@ -161,41 +172,67 @@ static void nla_channel_region_init(wmWindowManager *wm, ARegion *region)
   UI_view2d_region_reinit(&region->v2d, V2D_COMMONVIEW_LIST, region->winx, region->winy);
 
   /* own keymap */
-  /* own channels map first to override some channel keymaps */
-  keymap = WM_keymap_ensure(wm->defaultconf, "NLA Channels", SPACE_NLA, RGN_TYPE_WINDOW);
-  WM_event_add_keymap_handler_v2d_mask(&region->handlers, keymap);
+  /* own tracks map first to override some track keymaps */
+  keymap = WM_keymap_ensure(wm->runtime->defaultconf, "NLA Tracks", SPACE_NLA, RGN_TYPE_WINDOW);
+  WM_event_add_keymap_handler_poll(
+      &region->runtime->handlers, keymap, WM_event_handler_region_v2d_mask_no_marker_poll);
   /* now generic channels map for everything else that can apply */
-  keymap = WM_keymap_ensure(wm->defaultconf, "Animation Channels", SPACE_EMPTY, RGN_TYPE_WINDOW);
-  WM_event_add_keymap_handler_v2d_mask(&region->handlers, keymap);
+  keymap = WM_keymap_ensure(
+      wm->runtime->defaultconf, "Animation Channels", SPACE_EMPTY, RGN_TYPE_WINDOW);
+  WM_event_add_keymap_handler_v2d_mask(&region->runtime->handlers, keymap);
 
-  keymap = WM_keymap_ensure(wm->defaultconf, "NLA Generic", SPACE_NLA, RGN_TYPE_WINDOW);
-  WM_event_add_keymap_handler_v2d_mask(&region->handlers, keymap);
+  keymap = WM_keymap_ensure(wm->runtime->defaultconf, "NLA Generic", SPACE_NLA, RGN_TYPE_WINDOW);
+  WM_event_add_keymap_handler_v2d_mask(&region->runtime->handlers, keymap);
 }
 
 /* draw entirely, view changes should be handled here */
-static void nla_channel_region_draw(const bContext *C, ARegion *region)
+static void nla_track_region_draw(const bContext *C, ARegion *region)
 {
   bAnimContext ac;
-  View2D *v2d = &region->v2d;
+  if (!ANIM_animdata_get_context(C, &ac)) {
+    return;
+  }
 
   /* clear and setup matrix */
   UI_ThemeClearColor(TH_BACK);
 
+  ListBase anim_data = {nullptr, nullptr};
+
+  SpaceNla *snla = reinterpret_cast<SpaceNla *>(ac.sl);
+  View2D *v2d = &region->v2d;
+
+  const eAnimFilter_Flags filter = (ANIMFILTER_DATA_VISIBLE | ANIMFILTER_LIST_VISIBLE |
+                                    ANIMFILTER_LIST_CHANNELS | ANIMFILTER_FCURVESONLY);
+  const size_t item_count = ANIM_animdata_filter(
+      &ac, &anim_data, filter, ac.data, eAnimCont_Types(ac.datatype));
+
+  /* Recalculate the height of the track list.
+   * Needs to be done before the call to #UI_view2d_view_ortho. */
+  int height = NLATRACK_TOT_HEIGHT(&ac, item_count);
+  /* Add padding for the collapsed redo panel. */
+  height += HEADERY;
+  if (!BLI_listbase_is_empty(ED_context_get_markers(C))) {
+    height += (UI_MARKER_MARGIN_Y - NLATRACK_STEP(snla));
+  }
+  v2d->tot.ymin = -height;
+  UI_view2d_curRect_clamp_y(v2d);
+
   UI_view2d_view_ortho(v2d);
 
-  /* data */
-  if (ANIM_animdata_get_context(C, &ac)) {
-    draw_nla_channel_list(C, &ac, region);
-  }
+  draw_nla_track_list(C, &ac, region, anim_data);
 
-  /* channel filter next to scrubbing area */
+  /* track filter next to scrubbing area */
   ED_time_scrub_channel_search_draw(C, region, ac.ads);
 
   /* reset view matrix */
   UI_view2d_view_restore(C);
 
   /* scrollers */
-  UI_view2d_scrollers_draw(v2d, nullptr);
+  if (region->winy > UI_ANIM_MINY) {
+    UI_view2d_scrollers_draw(v2d, nullptr);
+  }
+
+  ANIM_animdata_freelist(&anim_data);
 }
 
 /* add handlers, stuff you only do once or on area/region changes */
@@ -206,10 +243,10 @@ static void nla_main_region_init(wmWindowManager *wm, ARegion *region)
   UI_view2d_region_reinit(&region->v2d, V2D_COMMONVIEW_CUSTOM, region->winx, region->winy);
 
   /* own keymap */
-  keymap = WM_keymap_ensure(wm->defaultconf, "NLA Editor", SPACE_NLA, RGN_TYPE_WINDOW);
-  WM_event_add_keymap_handler_v2d_mask(&region->handlers, keymap);
-  keymap = WM_keymap_ensure(wm->defaultconf, "NLA Generic", SPACE_NLA, RGN_TYPE_WINDOW);
-  WM_event_add_keymap_handler(&region->handlers, keymap);
+  keymap = WM_keymap_ensure(wm->runtime->defaultconf, "NLA Editor", SPACE_NLA, RGN_TYPE_WINDOW);
+  WM_event_add_keymap_handler_v2d_mask(&region->runtime->handlers, keymap);
+  keymap = WM_keymap_ensure(wm->runtime->defaultconf, "NLA Generic", SPACE_NLA, RGN_TYPE_WINDOW);
+  WM_event_add_keymap_handler(&region->runtime->handlers, keymap);
 }
 
 static void nla_main_region_draw(const bContext *C, ARegion *region)
@@ -220,18 +257,25 @@ static void nla_main_region_draw(const bContext *C, ARegion *region)
   bAnimContext ac;
   View2D *v2d = &region->v2d;
 
+  const int min_height = UI_ANIM_MINY;
+
   /* clear and setup matrix */
   UI_ThemeClearColor(TH_BACK);
 
   UI_view2d_view_ortho(v2d);
 
   /* time grid */
-  UI_view2d_draw_lines_x__discrete_frames_or_seconds(v2d, scene, snla->flag & SNLA_DRAWTIME, true);
+  if (region->winy > min_height) {
+    UI_view2d_draw_lines_x__discrete_frames_or_seconds(
+        v2d, scene, snla->flag & SNLA_DRAWTIME, true);
+  }
 
   ED_region_draw_cb_draw(C, region, REGION_DRAW_PRE_VIEW);
 
   /* start and end frame */
-  ANIM_draw_framerange(scene, v2d);
+  if (region->winy > min_height) {
+    ANIM_draw_framerange(scene, v2d);
+  }
 
   /* data */
   if (ANIM_animdata_get_context(C, &ac)) {
@@ -245,13 +289,13 @@ static void nla_main_region_draw(const bContext *C, ARegion *region)
   /* markers */
   UI_view2d_view_orthoSpecial(region, v2d, true);
   int marker_draw_flag = DRAW_MARKERS_MARGIN;
-  if (snla->flag & SNLA_SHOW_MARKERS) {
+  if (ED_markers_region_visible(CTX_wm_area(C), region)) {
     ED_markers_draw(C, marker_draw_flag);
   }
 
   /* preview range */
   UI_view2d_view_ortho(v2d);
-  ANIM_draw_previewrange(C, v2d, 0);
+  ANIM_draw_previewrange(scene, v2d, 0);
 
   /* callback */
   UI_view2d_view_ortho(v2d);
@@ -260,7 +304,8 @@ static void nla_main_region_draw(const bContext *C, ARegion *region)
   /* reset view matrix */
   UI_view2d_view_restore(C);
 
-  ED_time_scrub_draw(region, scene, snla->flag & SNLA_DRAWTIME, true);
+  const int fps = round_db_to_int(scene->frames_per_second());
+  ED_time_scrub_draw(region, scene, snla->flag & SNLA_DRAWTIME, true, fps);
 }
 
 static void nla_main_region_draw_overlay(const bContext *C, ARegion *region)
@@ -271,10 +316,13 @@ static void nla_main_region_draw_overlay(const bContext *C, ARegion *region)
   View2D *v2d = &region->v2d;
 
   /* scrubbing region */
-  ED_time_scrub_draw_current_frame(region, scene, snla->flag & SNLA_DRAWTIME);
+  ED_time_scrub_draw_current_frame(
+      region, scene, snla->flag & SNLA_DRAWTIME, region->winy >= UI_ANIM_MINY);
 
   /* scrollers */
-  UI_view2d_scrollers_draw(v2d, nullptr);
+  if (region->winy >= UI_ANIM_MINY) {
+    UI_view2d_scrollers_draw(v2d, nullptr);
+  }
 }
 
 /* add handlers, stuff you only do once or on area/region changes */
@@ -288,6 +336,28 @@ static void nla_header_region_draw(const bContext *C, ARegion *region)
   ED_region_header(C, region);
 }
 
+static void nla_footer_region_listener(const wmRegionListenerParams *params)
+{
+  ARegion *region = params->region;
+  const wmNotifier *wmn = params->notifier;
+
+  /* context changes */
+  switch (wmn->category) {
+    case NC_SCREEN:
+      if (wmn->data == ND_ANIMPLAY) {
+        ED_region_tag_redraw(region);
+      }
+      break;
+    case NC_SCENE:
+      switch (wmn->data) {
+        case ND_FRAME:
+          ED_region_tag_redraw(region);
+          break;
+      }
+      break;
+  }
+}
+
 /* add handlers, stuff you only do once or on area/region changes */
 static void nla_buttons_region_init(wmWindowManager *wm, ARegion *region)
 {
@@ -295,8 +365,8 @@ static void nla_buttons_region_init(wmWindowManager *wm, ARegion *region)
 
   ED_region_panels_init(wm, region);
 
-  keymap = WM_keymap_ensure(wm->defaultconf, "NLA Generic", SPACE_NLA, RGN_TYPE_WINDOW);
-  WM_event_add_keymap_handler_v2d_mask(&region->handlers, keymap);
+  keymap = WM_keymap_ensure(wm->runtime->defaultconf, "NLA Generic", SPACE_NLA, RGN_TYPE_WINDOW);
+  WM_event_add_keymap_handler_v2d_mask(&region->runtime->handlers, keymap);
 }
 
 static void nla_buttons_region_draw(const bContext *C, ARegion *region)
@@ -430,13 +500,7 @@ static void nla_main_region_message_subscribe(const wmRegionMessageSubscribePara
   }
 }
 
-static void nla_main_region_view2d_changed(const bContext * /*C*/, ARegion *region)
-{
-  View2D *v2d = &region->v2d;
-  UI_view2d_curRect_clamp_y(v2d);
-}
-
-static void nla_channel_region_listener(const wmRegionListenerParams *params)
+static void nla_track_region_listener(const wmRegionListenerParams *params)
 {
   ARegion *region = params->region;
   const wmNotifier *wmn = params->notifier;
@@ -450,6 +514,7 @@ static void nla_channel_region_listener(const wmRegionListenerParams *params)
       switch (wmn->data) {
         case ND_OB_ACTIVE:
         case ND_LAYER_CONTENT:
+        case ND_FRAME:
         case ND_OB_SELECT:
           ED_region_tag_redraw(region);
           break;
@@ -478,7 +543,7 @@ static void nla_channel_region_listener(const wmRegionListenerParams *params)
   }
 }
 
-static void nla_channel_region_message_subscribe(const wmRegionMessageSubscribeParams *params)
+static void nla_track_region_message_subscribe(const wmRegionMessageSubscribeParams *params)
 {
   wmMsgBus *mbus = params->message_bus;
   ARegion *region = params->region;
@@ -488,11 +553,10 @@ static void nla_channel_region_message_subscribe(const wmRegionMessageSubscribeP
   msg_sub_value_region_tag_redraw.user_data = region;
   msg_sub_value_region_tag_redraw.notify = ED_region_do_msg_notify_tag_redraw;
 
-  /* All dopesheet filter settings, etc. affect the drawing of this editor,
-   * so just whitelist the entire struct for updates
-   */
+  /* All dope-sheet filter settings, etc. affect the drawing of this editor,
+   * so just whitelist the entire struct for updates. */
   {
-    wmMsgParams_RNA msg_key_params = {{nullptr}};
+    wmMsgParams_RNA msg_key_params = {{}};
     StructRNA *type_array[] = {
         &RNA_DopeSheet,
     };
@@ -546,17 +610,18 @@ static void nla_listener(const wmSpaceTypeListenerParams *params)
   }
 }
 
-static void nla_id_remap(ScrArea * /*area*/, SpaceLink *slink, const IDRemapper *mappings)
+static void nla_id_remap(ScrArea * /*area*/,
+                         SpaceLink *slink,
+                         const blender::bke::id::IDRemapper &mappings)
 {
   SpaceNla *snla = reinterpret_cast<SpaceNla *>(slink);
 
   if (snla->ads == nullptr) {
     return;
   }
-  BKE_id_remapper_apply(
-      mappings, reinterpret_cast<ID **>(&snla->ads->filter_grp), ID_REMAP_APPLY_DEFAULT);
-  BKE_id_remapper_apply(
-      mappings, reinterpret_cast<ID **>(&snla->ads->source), ID_REMAP_APPLY_DEFAULT);
+
+  mappings.apply(reinterpret_cast<ID **>(&snla->ads->filter_grp), ID_REMAP_APPLY_DEFAULT);
+  mappings.apply((&snla->ads->source), ID_REMAP_APPLY_DEFAULT);
 }
 
 static void nla_foreach_id(SpaceLink *space_link, LibraryForeachIDData *data)
@@ -568,14 +633,14 @@ static void nla_foreach_id(SpaceLink *space_link, LibraryForeachIDData *data)
     return;
   }
 
-  BKE_LIB_FOREACHID_PROCESS_ID(data, snla->ads->source, IDWALK_CB_NOP);
-  BKE_LIB_FOREACHID_PROCESS_IDSUPER(data, snla->ads->filter_grp, IDWALK_CB_NOP);
+  BKE_LIB_FOREACHID_PROCESS_ID(data, snla->ads->source, IDWALK_CB_DIRECT_WEAK_LINK);
+  BKE_LIB_FOREACHID_PROCESS_IDSUPER(data, snla->ads->filter_grp, IDWALK_CB_DIRECT_WEAK_LINK);
 }
 
 static void nla_space_blend_read_data(BlendDataReader *reader, SpaceLink *sl)
 {
   SpaceNla *snla = reinterpret_cast<SpaceNla *>(sl);
-  BLO_read_data_address(reader, &snla->ads);
+  BLO_read_struct(reader, bDopeSheet, &snla->ads);
 }
 
 static void nla_space_blend_write(BlendWriter *writer, SpaceLink *sl)
@@ -590,11 +655,11 @@ static void nla_space_blend_write(BlendWriter *writer, SpaceLink *sl)
 
 void ED_spacetype_nla()
 {
-  SpaceType *st = MEM_cnew<SpaceType>("spacetype nla");
+  std::unique_ptr<SpaceType> st = std::make_unique<SpaceType>();
   ARegionType *art;
 
   st->spaceid = SPACE_NLA;
-  STRNCPY(st->name, "NLA");
+  STRNCPY_UTF8(st->name, "NLA");
 
   st->create = nla_create;
   st->free = nla_free;
@@ -610,20 +675,19 @@ void ED_spacetype_nla()
   st->blend_write = nla_space_blend_write;
 
   /* regions: main window */
-  art = MEM_cnew<ARegionType>("spacetype nla region");
+  art = MEM_callocN<ARegionType>("spacetype nla region");
   art->regionid = RGN_TYPE_WINDOW;
   art->init = nla_main_region_init;
   art->draw = nla_main_region_draw;
   art->draw_overlay = nla_main_region_draw_overlay;
   art->listener = nla_main_region_listener;
   art->message_subscribe = nla_main_region_message_subscribe;
-  art->on_view2d_changed = nla_main_region_view2d_changed;
   art->keymapflag = ED_KEYMAP_VIEW2D | ED_KEYMAP_ANIMATION | ED_KEYMAP_FRAMES;
 
   BLI_addhead(&st->regiontypes, art);
 
   /* regions: header */
-  art = MEM_cnew<ARegionType>("spacetype nla region");
+  art = MEM_callocN<ARegionType>("spacetype nla region");
   art->regionid = RGN_TYPE_HEADER;
   art->prefsizey = HEADERY;
   art->keymapflag = ED_KEYMAP_UI | ED_KEYMAP_VIEW2D | ED_KEYMAP_FRAMES | ED_KEYMAP_HEADER;
@@ -633,24 +697,36 @@ void ED_spacetype_nla()
 
   BLI_addhead(&st->regiontypes, art);
 
-  /* regions: channels */
-  art = MEM_cnew<ARegionType>("spacetype nla region");
+  /* regions: footer */
+  art = MEM_callocN<ARegionType>("spacetype nla region");
+  art->regionid = RGN_TYPE_FOOTER;
+  art->prefsizey = HEADERY;
+  art->keymapflag = ED_KEYMAP_UI | ED_KEYMAP_VIEW2D | ED_KEYMAP_FOOTER;
+
+  art->init = nla_header_region_init;
+  art->draw = nla_header_region_draw;
+  art->listener = nla_footer_region_listener;
+
+  BLI_addhead(&st->regiontypes, art);
+
+  /* regions: tracks */
+  art = MEM_callocN<ARegionType>("spacetype nla region");
   art->regionid = RGN_TYPE_CHANNELS;
   art->prefsizex = 200;
   art->keymapflag = ED_KEYMAP_UI | ED_KEYMAP_VIEW2D | ED_KEYMAP_FRAMES;
 
-  art->init = nla_channel_region_init;
-  art->draw = nla_channel_region_draw;
-  art->listener = nla_channel_region_listener;
-  art->message_subscribe = nla_channel_region_message_subscribe;
+  art->init = nla_track_region_init;
+  art->draw = nla_track_region_draw;
+  art->listener = nla_track_region_listener;
+  art->message_subscribe = nla_track_region_message_subscribe;
 
   BLI_addhead(&st->regiontypes, art);
 
   /* regions: UI buttons */
-  art = MEM_cnew<ARegionType>("spacetype nla region");
+  art = MEM_callocN<ARegionType>("spacetype nla region");
   art->regionid = RGN_TYPE_UI;
   art->prefsizex = UI_SIDEBAR_PANEL_WIDTH;
-  art->keymapflag = ED_KEYMAP_UI;
+  art->keymapflag = ED_KEYMAP_UI | ED_KEYMAP_FRAMES;
   art->listener = nla_region_listener;
   art->init = nla_buttons_region_init;
   art->draw = nla_buttons_region_draw;
@@ -662,5 +738,5 @@ void ED_spacetype_nla()
   art = ED_area_type_hud(st->spaceid);
   BLI_addhead(&st->regiontypes, art);
 
-  BKE_spacetype_register(st);
+  BKE_spacetype_register(std::move(st));
 }
