@@ -8,24 +8,34 @@
 
 #include <array>
 #include <cstdlib>
+#include <fmt/format.h>
 
+#include "DNA_collection_types.h"
+#include "DNA_image_types.h"
+#include "DNA_material_types.h"
 #include "DNA_node_types.h"
+#include "DNA_object_types.h"
 #include "DNA_windowmanager_types.h"
 
 #include "BLI_lasso_2d.hh"
 #include "BLI_listbase.h"
 #include "BLI_math_vector.h"
 #include "BLI_rect.h"
+#include "BLI_resource_scope.hh"
 #include "BLI_string.h"
 #include "BLI_string_utf8.h"
 #include "BLI_utildefines.h"
 
 #include "BKE_context.hh"
+#include "BKE_idtype.hh"
+#include "BKE_lib_id.hh"
 #include "BKE_main.hh"
+#include "BKE_main_invariants.hh"
 #include "BKE_node.hh"
 #include "BKE_node_legacy_types.hh"
 #include "BKE_node_runtime.hh"
 #include "BKE_node_tree_update.hh"
+#include "BKE_screen.hh"
 #include "BKE_viewer_path.hh"
 #include "BKE_workspace.hh"
 
@@ -41,13 +51,14 @@
 #include "WM_api.hh"
 #include "WM_types.hh"
 
-#include "UI_interface.hh"
 #include "UI_resources.hh"
 #include "UI_string_search.hh"
 #include "UI_view2d.hh"
 
 #include "DEG_depsgraph.hh"
 
+#include "BLT_translation.hh"
+
 #include "node_intern.hh" /* own include */
 
 namespace blender::ed::space_node {
@@ -125,7 +136,7 @@ static bool node_frame_select_isect_mouse(const SpaceNode &snode,
   return false;
 }
 
-static bNode *node_under_mouse_select(const SpaceNode &snode, const float2 mouse)
+bNode *node_under_mouse_get(const SpaceNode &snode, const float2 mouse)
 {
   for (bNode *node : tree_draw_order_calc_nodes_reversed(*snode.edittree)) {
     switch (node->type_legacy) {
@@ -148,7 +159,7 @@ static bNode *node_under_mouse_select(const SpaceNode &snode, const float2 mouse
 
 static bool is_position_over_node_or_socket(SpaceNode &snode, ARegion &region, const float2 &mouse)
 {
-  if (node_under_mouse_select(snode, mouse)) {
+  if (node_under_mouse_get(snode, mouse)) {
     return true;
   }
   if (node_find_indicated_socket(snode, region, mouse, SOCK_IN | SOCK_OUT)) {
@@ -221,7 +232,7 @@ bool node_deselect_all(bNodeTree &node_tree)
 {
   bool changed = false;
   for (bNode *node : node_tree.all_nodes()) {
-    changed |= bke::node_set_selected(node, false);
+    changed |= bke::node_set_selected(*node, false);
   }
   return changed;
 }
@@ -328,7 +339,7 @@ static bool node_select_grouped_type(bNodeTree &node_tree, bNode &node_act)
   for (bNode *node : node_tree.all_nodes()) {
     if ((node->flag & SELECT) == 0) {
       if (node->type_legacy == node_act.type_legacy) {
-        bke::node_set_selected(node, true);
+        bke::node_set_selected(*node, true);
         changed = true;
       }
     }
@@ -342,7 +353,7 @@ static bool node_select_grouped_color(bNodeTree &node_tree, bNode &node_act)
   for (bNode *node : node_tree.all_nodes()) {
     if ((node->flag & SELECT) == 0) {
       if (compare_v3v3(node->color, node_act.color, 0.005f)) {
-        bke::node_set_selected(node, true);
+        bke::node_set_selected(*node, true);
         changed = true;
       }
     }
@@ -383,7 +394,7 @@ static bool node_select_grouped_name(bNodeTree &node_tree, bNode &node_act, cons
         (!from_right && (pref_len_act == pref_len_curr) &&
          STREQLEN(node_act.name, node->name, pref_len_act)))
     {
-      bke::node_set_selected(node, true);
+      bke::node_set_selected(*node, true);
       changed = true;
     }
   }
@@ -398,11 +409,11 @@ enum {
   NODE_SELECT_GROUPED_SUFIX = 3,
 };
 
-static int node_select_grouped_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus node_select_grouped_exec(bContext *C, wmOperator *op)
 {
   SpaceNode &snode = *CTX_wm_space_node(C);
   bNodeTree &node_tree = *snode.edittree;
-  bNode *node_act = bke::node_get_active(snode.edittree);
+  bNode *node_act = bke::node_get_active(*snode.edittree);
 
   if (node_act == nullptr) {
     return OPERATOR_CANCELLED;
@@ -415,7 +426,7 @@ static int node_select_grouped_exec(bContext *C, wmOperator *op)
   if (!extend) {
     node_deselect_all(node_tree);
   }
-  bke::node_set_selected(node_act, true);
+  bke::node_set_selected(*node_act, true);
 
   switch (type) {
     case NODE_SELECT_GROUPED_TYPE:
@@ -459,7 +470,7 @@ void NODE_OT_select_grouped(wmOperatorType *ot)
   ot->description = "Select nodes with similar properties";
   ot->idname = "NODE_OT_select_grouped";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->invoke = WM_menu_invoke;
   ot->exec = node_select_grouped_exec;
   ot->poll = ED_operator_node_active;
@@ -495,10 +506,10 @@ void node_select_single(bContext &C, bNode &node)
 
   for (bNode *node_iter : node_tree.all_nodes()) {
     if (node_iter != &node) {
-      bke::node_set_selected(node_iter, false);
+      bke::node_set_selected(*node_iter, false);
     }
   }
-  bke::node_set_selected(&node, true);
+  bke::node_set_selected(node, true);
 
   ED_node_set_active(bmain, &snode, &node_tree, &node, &active_texture_changed);
   ED_node_set_active_viewer_key(&snode);
@@ -511,10 +522,75 @@ void node_select_single(bContext &C, bNode &node)
   WM_event_add_notifier(&C, NC_NODE | NA_SELECTED, nullptr);
 }
 
+static const bNodeSocket *find_socket_at_mouse_y(const Span<const bNodeSocket *> sockets,
+                                                 const float view_y)
+{
+  const bNodeSocket *best_socket = nullptr;
+  float best_distance = FLT_MAX;
+  for (const bNodeSocket *socket : sockets) {
+    if (!socket->is_icon_visible()) {
+      continue;
+    }
+    const float socket_y = socket->runtime->location.y;
+    const float distance = math::distance(socket_y, view_y);
+    if (distance < best_distance) {
+      best_distance = distance;
+      best_socket = socket;
+    }
+  }
+  return best_socket;
+}
+
+static void activate_interface_socket(bNodeTree &tree, bNodeTreeInterfaceSocket &io_socket)
+{
+  bNodeTreeInterfacePanel &io_panel = *tree.tree_interface.find_item_parent(io_socket.item, true);
+  bNodeTreeInterfaceItem *item_to_activate = nullptr;
+  if (io_panel.header_toggle_socket() == &io_socket) {
+    item_to_activate = &io_panel.item;
+  }
+  else {
+    item_to_activate = &io_socket.item;
+  }
+  tree.tree_interface.active_item_set(item_to_activate);
+}
+
+static void handle_group_input_node_selection(bNodeTree &tree,
+                                              const bNode &group_input_node,
+                                              const float2 &cursor)
+{
+
+  tree.ensure_topology_cache();
+  tree.ensure_interface_cache();
+  const bNodeSocket *indicated_socket = find_socket_at_mouse_y(
+      group_input_node.output_sockets().drop_back(1), cursor.y);
+  if (!indicated_socket) {
+    return;
+  }
+  const int group_input_i = indicated_socket->index();
+  bNodeTreeInterfaceSocket &io_socket = *tree.interface_inputs()[group_input_i];
+  activate_interface_socket(tree, io_socket);
+}
+
+static void handle_group_output_node_selection(bNodeTree &tree,
+                                               const bNode &group_output_node,
+                                               const float2 &cursor)
+{
+  tree.ensure_topology_cache();
+  tree.ensure_interface_cache();
+  const bNodeSocket *indicated_socket = find_socket_at_mouse_y(
+      group_output_node.input_sockets().drop_back(1), cursor.y);
+  if (!indicated_socket) {
+    return;
+  }
+  const int group_output_i = indicated_socket->index();
+  bNodeTreeInterfaceSocket &io_socket = *tree.interface_outputs()[group_output_i];
+  activate_interface_socket(tree, io_socket);
+}
+
 static bool node_mouse_select(bContext *C,
                               wmOperator *op,
                               const int2 mval,
-                              SelectPick_Params *params)
+                              const SelectPick_Params &params)
 {
   Main &bmain = *CTX_data_main(C);
   SpaceNode &snode = *CTX_wm_space_node(C);
@@ -527,7 +603,7 @@ static bool node_mouse_select(bContext *C,
   bNodeSocket *sock = nullptr;
 
   /* Always do socket_select when extending selection. */
-  const bool socket_select = (params->sel_op == SEL_OP_XOR) ||
+  const bool socket_select = (params.sel_op == SEL_OP_XOR) ||
                              RNA_boolean_get(op->ptr, "socket_select");
   bool changed = false;
   bool found = false;
@@ -540,7 +616,7 @@ static bool node_mouse_select(bContext *C,
   /* First do socket selection, these generally overlap with nodes. */
   if (socket_select) {
     /* NOTE: unlike nodes #SelectPick_Params isn't fully supported. */
-    const bool extend = (params->sel_op == SEL_OP_XOR);
+    const bool extend = (params.sel_op == SEL_OP_XOR);
     sock = node_find_indicated_socket(snode, region, cursor, SOCK_IN);
     if (sock) {
       node = &sock->owner_node();
@@ -596,36 +672,36 @@ static bool node_mouse_select(bContext *C,
   if (!sock) {
 
     /* Find the closest visible node. */
-    node = node_under_mouse_select(snode, cursor);
+    node = node_under_mouse_get(snode, cursor);
     found = (node != nullptr);
     node_was_selected = node && (node->flag & SELECT);
 
-    if (params->sel_op == SEL_OP_SET) {
-      if ((found && params->select_passthrough) && (node->flag & SELECT)) {
+    if (params.sel_op == SEL_OP_SET) {
+      if ((found && params.select_passthrough) && (node->flag & SELECT)) {
         found = false;
       }
-      else if (found || params->deselect_all) {
+      else if (found || params.deselect_all) {
         /* Deselect everything. */
         changed = node_deselect_all(node_tree);
       }
     }
 
     if (found) {
-      switch (params->sel_op) {
+      switch (params.sel_op) {
         case SEL_OP_ADD:
-          bke::node_set_selected(node, true);
+          bke::node_set_selected(*node, true);
           break;
         case SEL_OP_SUB:
-          bke::node_set_selected(node, false);
+          bke::node_set_selected(*node, false);
           break;
         case SEL_OP_XOR: {
           /* Check active so clicking on an inactive node activates it. */
           bool is_selected = (node->flag & NODE_SELECT) && (node->flag & NODE_ACTIVE);
-          bke::node_set_selected(node, !is_selected);
+          bke::node_set_selected(*node, !is_selected);
           break;
         }
         case SEL_OP_SET:
-          bke::node_set_selected(node, true);
+          bke::node_set_selected(*node, true);
           break;
         case SEL_OP_AND:
           /* Doesn't make sense for picking. */
@@ -633,6 +709,13 @@ static bool node_mouse_select(bContext *C,
           break;
       }
 
+      if (node->is_group_input()) {
+        handle_group_input_node_selection(node_tree, *node, cursor);
+      }
+      if (node->is_group_output()) {
+        handle_group_output_node_selection(node_tree, *node, cursor);
+      }
+
       changed = true;
     }
   }
@@ -641,6 +724,15 @@ static bool node_mouse_select(bContext *C,
     if (node == nullptr) {
       /* Disable existing active viewer. */
       WorkSpace *workspace = CTX_wm_workspace(C);
+      if (const std::optional<viewer_path::ViewerPathForGeometryNodesViewer> parsed_path =
+              viewer_path::parse_geometry_nodes_viewer(workspace->viewer_path))
+      {
+        /* The object needs to be reevaluated, because the viewer path is changed which means that
+         * the object may generate different viewer geometry as a side effect. */
+        Depsgraph *depsgraph = CTX_data_depsgraph_pointer(C);
+        DEG_id_tag_update_for_side_effect_request(
+            depsgraph, &parsed_path->object->id, ID_RECALC_GEOMETRY);
+      }
       BKE_viewer_path_clear(&workspace->viewer_path);
       WM_event_add_notifier(C, NC_VIEWER_PATH, nullptr);
     }
@@ -652,7 +744,7 @@ static bool node_mouse_select(bContext *C,
 
   bool active_texture_changed = false;
   bool viewer_node_changed = false;
-  if ((node != nullptr) && (node_was_selected == false || params->select_passthrough == false)) {
+  if ((node != nullptr) && (node_was_selected == false || params.select_passthrough == false)) {
     viewer_node_changed = (node->flag & NODE_DO_OUTPUT) == 0 &&
                           node->type_legacy == GEO_NODE_VIEWER;
     ED_node_set_active(&bmain, &snode, snode.edittree, node, &active_texture_changed);
@@ -671,20 +763,21 @@ static bool node_mouse_select(bContext *C,
   WM_event_add_notifier(C, NC_NODE | NA_SELECTED, nullptr);
   WM_event_add_notifier(C, NC_NODE | ND_NODE_GIZMO, nullptr);
 
+  BKE_main_ensure_invariants(bmain, node_tree.id);
+
   return true;
 }
 
-static int node_select_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus node_select_exec(bContext *C, wmOperator *op)
 {
   /* Get settings from RNA properties for operator. */
   int2 mval;
   RNA_int_get_array(op->ptr, "location", mval);
 
-  SelectPick_Params params = {};
-  ED_select_pick_params_from_operator(op->ptr, &params);
+  const SelectPick_Params params = ED_select_pick_params_from_operator(op->ptr);
 
   /* Perform the selection. */
-  const bool changed = node_mouse_select(C, op, mval, &params);
+  const bool changed = node_mouse_select(C, op, mval, params);
 
   if (changed) {
     return OPERATOR_PASS_THROUGH | OPERATOR_FINISHED;
@@ -693,11 +786,11 @@ static int node_select_exec(bContext *C, wmOperator *op)
   return OPERATOR_PASS_THROUGH | OPERATOR_CANCELLED;
 }
 
-static int node_select_invoke(bContext *C, wmOperator *op, const wmEvent *event)
+static wmOperatorStatus node_select_invoke(bContext *C, wmOperator *op, const wmEvent *event)
 {
   RNA_int_set_array(op->ptr, "location", event->mval);
 
-  const int retval = node_select_exec(C, op);
+  const wmOperatorStatus retval = node_select_exec(C, op);
 
   return WM_operator_flag_only_pass_through_on_press(retval, event);
 }
@@ -711,7 +804,7 @@ void NODE_OT_select(wmOperatorType *ot)
   ot->idname = "NODE_OT_select";
   ot->description = "Select the node under the cursor";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = node_select_exec;
   ot->invoke = node_select_invoke;
   ot->poll = ED_operator_node_active;
@@ -750,7 +843,7 @@ void NODE_OT_select(wmOperatorType *ot)
 /** \name Box Select Operator
  * \{ */
 
-static int node_box_select_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus node_box_select_exec(bContext *C, wmOperator *op)
 {
   SpaceNode &snode = *CTX_wm_space_node(C);
   bNodeTree &node_tree = *snode.edittree;
@@ -777,7 +870,7 @@ static int node_box_select_exec(bContext *C, wmOperator *op)
         if (BLI_rctf_isect(&rectf, &node->runtime->draw_bounds, nullptr) &&
             !BLI_rctf_inside_rctf(&frame_inside, &rectf))
         {
-          bke::node_set_selected(node, select);
+          bke::node_set_selected(*node, select);
           is_inside = true;
         }
         break;
@@ -789,18 +882,19 @@ static int node_box_select_exec(bContext *C, wmOperator *op)
     }
 
     if (is_inside) {
-      bke::node_set_selected(node, select);
+      bke::node_set_selected(*node, select);
     }
   }
 
   tree_draw_order_update(node_tree);
 
   WM_event_add_notifier(C, NC_NODE | NA_SELECTED, nullptr);
+  WM_event_add_notifier(C, NC_NODE | ND_NODE_GIZMO, nullptr);
 
   return OPERATOR_FINISHED;
 }
 
-static int node_box_select_invoke(bContext *C, wmOperator *op, const wmEvent *event)
+static wmOperatorStatus node_box_select_invoke(bContext *C, wmOperator *op, const wmEvent *event)
 {
   const bool tweak = RNA_boolean_get(op->ptr, "tweak");
 
@@ -818,7 +912,7 @@ void NODE_OT_select_box(wmOperatorType *ot)
   ot->idname = "NODE_OT_select_box";
   ot->description = "Use box selection to select nodes";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->invoke = node_box_select_invoke;
   ot->exec = node_box_select_exec;
   ot->modal = WM_gesture_box_modal;
@@ -846,7 +940,7 @@ void NODE_OT_select_box(wmOperatorType *ot)
 /** \name Circle Select Operator
  * \{ */
 
-static int node_circleselect_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus node_circleselect_exec(bContext *C, wmOperator *op)
 {
   SpaceNode *snode = CTX_wm_space_node(C);
   ARegion *region = CTX_wm_region(C);
@@ -883,13 +977,13 @@ static int node_circleselect_exec(bContext *C, wmOperator *op)
         if (BLI_rctf_isect_circle(&node->runtime->draw_bounds, offset, radius_adjusted) &&
             !BLI_rctf_isect_circle(&frame_inside, offset, radius_adjusted))
         {
-          bke::node_set_selected(node, select);
+          bke::node_set_selected(*node, select);
         }
         break;
       }
       default: {
         if (BLI_rctf_isect_circle(&node->runtime->draw_bounds, offset, radius / zoom)) {
-          bke::node_set_selected(node, select);
+          bke::node_set_selected(*node, select);
         }
         break;
       }
@@ -897,6 +991,7 @@ static int node_circleselect_exec(bContext *C, wmOperator *op)
   }
 
   WM_event_add_notifier(C, NC_NODE | NA_SELECTED, nullptr);
+  WM_event_add_notifier(C, NC_NODE | ND_NODE_GIZMO, nullptr);
 
   return OPERATOR_FINISHED;
 }
@@ -908,7 +1003,7 @@ void NODE_OT_select_circle(wmOperatorType *ot)
   ot->idname = "NODE_OT_select_circle";
   ot->description = "Use circle selection to select nodes";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->invoke = WM_gesture_circle_invoke;
   ot->exec = node_circleselect_exec;
   ot->modal = WM_gesture_circle_modal;
@@ -929,7 +1024,7 @@ void NODE_OT_select_circle(wmOperatorType *ot)
 /** \name Lasso Select Operator
  * \{ */
 
-static int node_lasso_select_invoke(bContext *C, wmOperator *op, const wmEvent *event)
+static wmOperatorStatus node_lasso_select_invoke(bContext *C, wmOperator *op, const wmEvent *event)
 {
   const bool tweak = RNA_boolean_get(op->ptr, "tweak");
 
@@ -975,7 +1070,7 @@ static bool do_lasso_select_node(bContext *C, const Span<int2> mcoords, eSelectO
         if (BLI_rctf_isect(&rectf, &node->runtime->draw_bounds, nullptr) &&
             !BLI_rctf_inside_rctf(&frame_inside, &rectf))
         {
-          bke::node_set_selected(node, select);
+          bke::node_set_selected(*node, select);
           changed = true;
         }
         break;
@@ -991,7 +1086,7 @@ static bool do_lasso_select_node(bContext *C, const Span<int2> mcoords, eSelectO
             BLI_rcti_isect_pt(&rect, screen_co.x, screen_co.y) &&
             BLI_lasso_is_point_inside(mcoords, screen_co.x, screen_co.y, INT_MAX))
         {
-          bke::node_set_selected(node, select);
+          bke::node_set_selected(*node, select);
           changed = true;
         }
         break;
@@ -1001,12 +1096,13 @@ static bool do_lasso_select_node(bContext *C, const Span<int2> mcoords, eSelectO
 
   if (changed) {
     WM_event_add_notifier(C, NC_NODE | NA_SELECTED, nullptr);
+    WM_event_add_notifier(C, NC_NODE | ND_NODE_GIZMO, nullptr);
   }
 
   return changed;
 }
 
-static int node_lasso_select_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus node_lasso_select_exec(bContext *C, wmOperator *op)
 {
   const Array<int2> mcoords = WM_gesture_lasso_path_to_array(C, op);
 
@@ -1028,7 +1124,7 @@ void NODE_OT_select_lasso(wmOperatorType *ot)
   ot->description = "Select nodes using lasso selection";
   ot->idname = "NODE_OT_select_lasso";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->invoke = node_lasso_select_invoke;
   ot->modal = WM_gesture_lasso_modal;
   ot->exec = node_lasso_select_exec;
@@ -1065,7 +1161,7 @@ static bool any_node_selected(const bNodeTree &node_tree)
   return false;
 }
 
-static int node_select_all_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus node_select_all_exec(bContext *C, wmOperator *op)
 {
   SpaceNode &snode = *CTX_wm_space_node(C);
   bNodeTree &node_tree = *snode.edittree;
@@ -1085,7 +1181,7 @@ static int node_select_all_exec(bContext *C, wmOperator *op)
   switch (action) {
     case SEL_SELECT:
       for (bNode *node : node_tree.all_nodes()) {
-        bke::node_set_selected(node, true);
+        bke::node_set_selected(*node, true);
       }
       break;
     case SEL_DESELECT:
@@ -1093,7 +1189,7 @@ static int node_select_all_exec(bContext *C, wmOperator *op)
       break;
     case SEL_INVERT:
       for (bNode *node : node_tree.all_nodes()) {
-        bke::node_set_selected(node, !(node->flag & SELECT));
+        bke::node_set_selected(*node, !(node->flag & SELECT));
       }
       break;
   }
@@ -1112,7 +1208,7 @@ void NODE_OT_select_all(wmOperatorType *ot)
   ot->description = "(De)select all nodes";
   ot->idname = "NODE_OT_select_all";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = node_select_all_exec;
   ot->poll = ED_operator_node_active;
 
@@ -1128,7 +1224,7 @@ void NODE_OT_select_all(wmOperatorType *ot)
 /** \name Select Linked To Operator
  * \{ */
 
-static int node_select_linked_to_exec(bContext *C, wmOperator * /*op*/)
+static wmOperatorStatus node_select_linked_to_exec(bContext *C, wmOperator * /*op*/)
 {
   SpaceNode &snode = *CTX_wm_space_node(C);
   bNodeTree &node_tree = *snode.edittree;
@@ -1146,7 +1242,7 @@ static int node_select_linked_to_exec(bContext *C, wmOperator * /*op*/)
         if (!input_socket->is_available()) {
           continue;
         }
-        bke::node_set_selected(&input_socket->owner_node(), true);
+        bke::node_set_selected(input_socket->owner_node(), true);
       }
     }
   }
@@ -1164,7 +1260,7 @@ void NODE_OT_select_linked_to(wmOperatorType *ot)
   ot->description = "Select nodes linked to the selected ones";
   ot->idname = "NODE_OT_select_linked_to";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = node_select_linked_to_exec;
   ot->poll = ED_operator_node_active;
 
@@ -1178,7 +1274,7 @@ void NODE_OT_select_linked_to(wmOperatorType *ot)
 /** \name Select Linked From Operator
  * \{ */
 
-static int node_select_linked_from_exec(bContext *C, wmOperator * /*op*/)
+static wmOperatorStatus node_select_linked_from_exec(bContext *C, wmOperator * /*op*/)
 {
   SpaceNode &snode = *CTX_wm_space_node(C);
   bNodeTree &node_tree = *snode.edittree;
@@ -1196,7 +1292,7 @@ static int node_select_linked_from_exec(bContext *C, wmOperator * /*op*/)
         if (!output_socket->is_available()) {
           continue;
         }
-        bke::node_set_selected(&output_socket->owner_node(), true);
+        bke::node_set_selected(output_socket->owner_node(), true);
       }
     }
   }
@@ -1214,7 +1310,7 @@ void NODE_OT_select_linked_from(wmOperatorType *ot)
   ot->description = "Select nodes linked from the selected ones";
   ot->idname = "NODE_OT_select_linked_from";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = node_select_linked_from_exec;
   ot->poll = ED_operator_node_active;
 
@@ -1233,12 +1329,12 @@ static bool nodes_are_same_type_for_select(const bNode &a, const bNode &b)
   return a.type_legacy == b.type_legacy;
 }
 
-static int node_select_same_type_step_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus node_select_same_type_step_exec(bContext *C, wmOperator *op)
 {
   SpaceNode *snode = CTX_wm_space_node(C);
   ARegion *region = CTX_wm_region(C);
   const bool prev = RNA_boolean_get(op->ptr, "prev");
-  bNode *active_node = bke::node_get_active(snode->edittree);
+  bNode *active_node = bke::node_get_active(*snode->edittree);
 
   if (active_node == nullptr) {
     return OPERATOR_CANCELLED;
@@ -1286,7 +1382,7 @@ void NODE_OT_select_same_type_step(wmOperatorType *ot)
   ot->description = "Activate and view same node type, step by step";
   ot->idname = "NODE_OT_select_same_type_step";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = node_select_same_type_step_exec;
   ot->poll = ED_operator_node_active;
 
@@ -1302,14 +1398,34 @@ void NODE_OT_select_same_type_step(wmOperatorType *ot)
 /** \name Find Node by Name Operator
  * \{ */
 
-static void node_find_create_label(const bNode *node, char *str, int str_maxncpy)
+static std::string node_find_create_node_label(const bNodeTree &ntree, const bNode &node)
 {
-  if (node->label[0]) {
-    BLI_snprintf(str, str_maxncpy, "%s (%s)", node->name, node->label);
+  std::string label = bke::node_label(ntree, node);
+  if (label == node.name) {
+    return label;
   }
-  else {
-    BLI_strncpy(str, node->name, str_maxncpy);
+  return fmt::format("{} ({})", label, node.name);
+}
+
+static std::string node_find_create_group_input_label(const bNode &node, const bNodeSocket &socket)
+{
+  return fmt::format("{}: \"{}\" ({})", TIP_("Input"), socket.name, node.name);
+}
+
+static std::string node_find_create_string_value(const bNode &node, const StringRef str)
+{
+  return fmt::format("{}: \"{}\" ({})", TIP_("String"), str, node.name);
+}
+
+static std::string node_find_create_data_block_value(const bNode &node, const ID &id)
+{
+  const IDTypeInfo *type = BKE_idtype_get_info_from_id(&id);
+  BLI_assert(type);
+  StringRef type_name = TIP_(type->name);
+  if (GS(id.name) == ID_NT) {
+    type_name = TIP_("Node Group");
   }
+  return fmt::format("{}: \"{}\" ({})", type_name, BKE_id_name(id), node.name);
 }
 
 /* Generic search invoke. */
@@ -1321,20 +1437,103 @@ static void node_find_update_fn(const bContext *C,
 {
   SpaceNode *snode = CTX_wm_space_node(C);
 
-  ui::string_search::StringSearch<bNode> search;
+  struct Item {
+    bNode *node;
+    std::string search_str;
+  };
 
+  ui::string_search::StringSearch<Item> search;
+  blender::ResourceScope scope;
+
+  auto add_data_block_item = [&](bNode &node, const ID *id) {
+    if (!id) {
+      return;
+    }
+    const StringRef search_str = scope.add_value(node_find_create_data_block_value(node, *id));
+    search.add(search_str, &scope.construct<Item>(Item{&node, search_str}));
+  };
+
+  const bNodeTree &ntree = *snode->edittree;
+  ntree.ensure_topology_cache();
   for (bNode *node : snode->edittree->all_nodes()) {
-    char name[256];
-    node_find_create_label(node, name, ARRAY_SIZE(name));
-    search.add(name, node);
+    const StringRef name = scope.add_value(node_find_create_node_label(ntree, *node));
+    search.add(name, &scope.construct<Item>(Item{node, name}));
+
+    if (node->is_type("FunctionNodeInputString")) {
+      const auto *storage = static_cast<const NodeInputString *>(node->storage);
+      const StringRef value_str = storage->string;
+      if (!value_str.is_empty()) {
+        const StringRef search_str = scope.add_value(
+            node_find_create_string_value(*node, value_str));
+        search.add(search_str, &scope.construct<Item>(Item{node, search_str}));
+      }
+    }
+    if (node->is_group_input()) {
+      for (const bNodeSocket *socket : node->output_sockets().drop_back(1)) {
+        if (!socket->is_directly_linked()) {
+          continue;
+        }
+        const StringRef search_str = scope.add_value(
+            node_find_create_group_input_label(*node, *socket));
+        search.add(search_str, &scope.construct<Item>(Item{node, search_str}));
+      }
+    }
+    if (node->id) {
+      /* Avoid showing referenced node group data-blocks twice. */
+      const bool skip_data_block =
+          node->is_group() &&
+          StringRef(bke::node_label(ntree, *node)).find(BKE_id_name(*node->id)) !=
+              StringRef::not_found;
+      if (!skip_data_block) {
+        add_data_block_item(*node, node->id);
+      }
+    }
+
+    for (const bNodeSocket *socket : node->input_sockets()) {
+      switch (socket->type) {
+        case SOCK_STRING: {
+          if (socket->is_logically_linked()) {
+            continue;
+          }
+          const bNodeSocketValueString *value =
+              socket->default_value_typed<bNodeSocketValueString>();
+          const StringRef value_str = value->value;
+          if (!value_str.is_empty()) {
+            const StringRef search_str = scope.add_value(
+                node_find_create_string_value(*node, value_str));
+            search.add(search_str, &scope.construct<Item>(Item{node, search_str}));
+          }
+          break;
+        }
+        case SOCK_OBJECT: {
+          add_data_block_item(
+              *node, id_cast<ID *>(socket->default_value_typed<bNodeSocketValueObject>()->value));
+          break;
+        }
+        case SOCK_MATERIAL: {
+          add_data_block_item(
+              *node,
+              id_cast<ID *>(socket->default_value_typed<bNodeSocketValueMaterial>()->value));
+          break;
+        }
+        case SOCK_COLLECTION: {
+          add_data_block_item(
+              *node,
+              id_cast<ID *>(socket->default_value_typed<bNodeSocketValueCollection>()->value));
+          break;
+        }
+        case SOCK_IMAGE: {
+          add_data_block_item(
+              *node, id_cast<ID *>(socket->default_value_typed<bNodeSocketValueImage>()->value));
+          break;
+        }
+      }
+    }
   }
 
-  const Vector<bNode *> filtered_nodes = search.query(str);
-
-  for (bNode *node : filtered_nodes) {
-    char name[256];
-    node_find_create_label(node, name, ARRAY_SIZE(name));
-    if (!UI_search_item_add(items, name, node, ICON_NONE, 0, 0)) {
+  const Vector<Item *> filtered_items = search.query(str);
+  for (const Item *item : filtered_items) {
+    if (!UI_search_item_add(items, item->search_str, item->node, ICON_NONE, 0, 0)) {
       break;
     }
   }
@@ -1362,46 +1561,33 @@ static uiBlock *node_find_menu(bContext *C, ARegion *region, void *arg_optype)
   uiBut *but;
   wmOperatorType *optype = (wmOperatorType *)arg_optype;
 
-  block = UI_block_begin(C, region, "_popup", UI_EMBOSS);
+  block = UI_block_begin(C, region, "_popup", ui::EmbossType::Emboss);
   UI_block_flag_enable(block, UI_BLOCK_LOOP | UI_BLOCK_MOVEMOUSE_QUIT | UI_BLOCK_SEARCH_MENU);
   UI_block_theme_style_set(block, UI_BLOCK_THEME_STYLE_POPUP);
 
-  but = uiDefSearchBut(block,
-                       search,
-                       0,
-                       ICON_VIEWZOOM,
-                       sizeof(search),
-                       10,
-                       10,
-                       UI_searchbox_size_x(),
-                       UI_UNIT_Y,
-                       "");
+  const int box_width = UI_searchbox_size_x_guess(C, node_find_update_fn, nullptr);
+
+  but = uiDefSearchBut(
+      block, search, 0, ICON_VIEWZOOM, sizeof(search), 0, 0, box_width, UI_UNIT_Y, "");
   UI_but_func_search_set(
       but, nullptr, node_find_update_fn, optype, false, nullptr, node_find_exec_fn, nullptr);
   UI_but_flag_enable(but, UI_BUT_ACTIVATE_ON_INIT);
 
   /* Fake button holds space for search items. */
-  uiDefBut(block,
-           UI_BTYPE_LABEL,
-           0,
-           "",
-           10,
-           10 - UI_searchbox_size_y(),
-           UI_searchbox_size_x(),
-           UI_searchbox_size_y(),
-           nullptr,
-           0,
-           0,
-           nullptr);
+  const int height = UI_searchbox_size_y() - UI_SEARCHBOX_BOUNDS;
+  uiDefBut(
+      block, ButType::Label, 0, "", 0, -height, box_width, height, nullptr, 0, 0, std::nullopt);
 
   /* Move it downwards, mouse over button. */
   std::array<int, 2> bounds_offset = {0, -UI_UNIT_Y};
-  UI_block_bounds_set_popup(block, 0.3f * U.widget_unit, bounds_offset.data());
+  UI_block_bounds_set_popup(block, UI_SEARCHBOX_BOUNDS, bounds_offset.data());
 
   return block;
 }
 
-static int node_find_node_invoke(bContext *C, wmOperator *op, const wmEvent * /*event*/)
+static wmOperatorStatus node_find_node_invoke(bContext *C,
+                                              wmOperator *op,
+                                              const wmEvent * /*event*/)
 {
   UI_popup_block_invoke(C, node_find_menu, op->type, nullptr);
   return OPERATOR_CANCELLED;
@@ -1414,7 +1600,7 @@ void NODE_OT_find_node(wmOperatorType *ot)
   ot->description = "Search for a node by name and focus and select it";
   ot->idname = "NODE_OT_find_node";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->invoke = node_find_node_invoke;
   ot->poll = ED_operator_node_active;
 
