@@ -6,11 +6,11 @@
  * \ingroup cmpnodes
  */
 
+#include "BLI_listbase.h"
+
 #include "BKE_context.hh"
 #include "BKE_lib_id.hh"
 
-#include "UI_interface.hh"
-
 #include "COM_node_operation.hh"
 
 #include "node_composite_util.hh"
@@ -21,7 +21,7 @@ namespace blender::nodes::node_composite_switchview_cc {
 
 static void node_declare(NodeDeclarationBuilder &b)
 {
-  b.add_output<decl::Color>(N_("Image"));
+  b.add_output<decl::Color>(N_("Image")).structure_type(StructureType::Dynamic);
 
   const bNode *node = b.node_or_null();
   if (node == nullptr) {
@@ -36,7 +36,9 @@ static void node_declare(NodeDeclarationBuilder &b)
       if (srv->viewflag & SCE_VIEW_DISABLE) {
         continue;
       }
-      b.add_input<decl::Color>(N_(srv->name)).default_value({0.0f, 0.0f, 0.0f, 1.0f});
+      b.add_input<decl::Color>(N_(srv->name))
+          .default_value({0.0f, 0.0f, 0.0f, 1.0f})
+          .structure_type(StructureType::Dynamic);
     }
   }
 }
@@ -63,13 +65,13 @@ class SwitchViewOperation : public NodeOperation {
 
     /* A context that is not multi view, pass the first input through as a fallback. */
     if (context().get_view_name().is_empty()) {
-      Result &input = get_input(node().input(0)->identifier);
-      input.pass_through(result);
+      const Result &input = get_input(node().input(0)->identifier);
+      result.share_data(input);
       return;
     }
 
-    Result &input = get_input(context().get_view_name());
-    input.pass_through(result);
+    const Result &input = get_input(context().get_view_name());
+    result.share_data(input);
   }
 };
 
@@ -80,7 +82,7 @@ static NodeOperation *get_compositor_operation(Context &context, DNode node)
 
 }  // namespace blender::nodes::node_composite_switchview_cc
 
-void register_node_type_cmp_switch_view()
+static void register_node_type_cmp_switch_view()
 {
   namespace file_ns = blender::nodes::node_composite_switchview_cc;
 
@@ -95,5 +97,6 @@ void register_node_type_cmp_switch_view()
   ntype.initfunc_api = file_ns::init_switch_view;
   ntype.get_compositor_operation = file_ns::get_compositor_operation;
 
-  blender::bke::node_register_type(&ntype);
+  blender::bke::node_register_type(ntype);
 }
+NOD_REGISTER_NODE(register_node_type_cmp_switch_view)
