@@ -205,7 +205,7 @@ static const PyC_StringEnumItems pygpu_depth_write_items[] = {
   "      - ``R16``\n" \
   "      - ``R11F_G11F_B10F``\n" \
   "      - ``DEPTH32F_STENCIL8``\n" \
-  "      - ``DEPTH24_STENCIL8``\n" \
+  "      - ``DEPTH24_STENCIL8`` (deprecated, use ``DEPTH32F_STENCIL8``)\n" \
   "      - ``SRGB8_A8``\n" \
   "      - ``RGB16F``\n" \
   "      - ``SRGB8_A8_DXT1``\n" \
@@ -215,7 +215,7 @@ static const PyC_StringEnumItems pygpu_depth_write_items[] = {
   "      - ``RGBA8_DXT3``\n" \
   "      - ``RGBA8_DXT5``\n" \
   "      - ``DEPTH_COMPONENT32F``\n" \
-  "      - ``DEPTH_COMPONENT24``\n" \
+  "      - ``DEPTH_COMPONENT24`` (deprecated, use ``DEPTH_COMPONENT32F``)\n" \
   "      - ``DEPTH_COMPONENT16``\n"
 extern const PyC_StringEnumItems pygpu_tex_format_items[];
 
@@ -353,7 +353,7 @@ PyDoc_STRVAR(
     pygpu_interface_info_name_doc,
     "Name of the interface block.\n"
     "\n"
-    ":type: str");
+    ":type: str\n");
 static PyObject *pygpu_interface_info_name_get(BPyGPUStageInterfaceInfo *self, void * /*closure*/)
 {
   StageInterfaceInfo *interface = reinterpret_cast<StageInterfaceInfo *>(self->interface);
@@ -582,7 +582,7 @@ static PyObject *pygpu_shader_info_vertex_out(BPyGPUShaderCreateInfo *self,
 PyDoc_STRVAR(
     /* Wrap. */
     pygpu_shader_info_fragment_out_doc,
-    ".. method:: fragment_out(slot, type, name, blend='NONE')\n"
+    ".. method:: fragment_out(slot, type, name, *, blend='NONE')\n"
     "\n"
     "   Specify a fragment output corresponding to a framebuffer target slot.\n"
     "\n"
@@ -724,13 +724,14 @@ static PyObject *pygpu_shader_info_uniform_buf(BPyGPUShaderCreateInfo *self, PyO
 PyDoc_STRVAR(
     /* Wrap. */
     pygpu_shader_info_image_doc,
-    ".. method:: image(slot, format, type, name, qualifiers={'NO_RESTRICT'})\n"
+    ".. method:: image(slot, format, type, name, *, qualifiers={'NO_RESTRICT'})\n"
     "\n"
     "   Specify an image resource used for arbitrary load and store operations.\n"
     "\n"
     "   :arg slot: The image resource index.\n"
     "   :type slot: int\n"
-    "   :arg format: The GPUTexture format that is passed to the shader. Possible values are:\n"
+    "   :arg format: The GPUTexture format that is passed to the shader. Possible "
+    "values are:\n"
     "\n" PYDOC_TEX_FORMAT_ITEMS
     "   :type format: str\n"
     "   :arg type: The data type describing how the image is to be read in the shader. "
@@ -790,13 +791,25 @@ static PyObject *pygpu_shader_info_image(BPyGPUShaderCreateInfo *self,
     return nullptr;
   }
 
+  if (pygpu_texformat.value_found == GPU_DEPTH24_STENCIL8_DEPRECATED) {
+    pygpu_texformat.value_found = int(blender::gpu::TextureFormat::SFLOAT_32_DEPTH_UINT_8);
+    PyErr_WarnEx(
+        PyExc_DeprecationWarning, "'DEPTH24_STENCIL8' is deprecated. Use 'DEPTH32F_STENCIL8'.", 1);
+  }
+  if (pygpu_texformat.value_found == GPU_DEPTH_COMPONENT24_DEPRECATED) {
+    pygpu_texformat.value_found = int(blender::gpu::TextureFormat::SFLOAT_32_DEPTH);
+    PyErr_WarnEx(PyExc_DeprecationWarning,
+                 "'DEPTH_COMPONENT24' is deprecated. Use 'DEPTH_COMPONENT32F'.",
+                 1);
+  }
+
 #  ifdef USE_GPU_PY_REFERENCES
   PyList_Append(self->references, PyTuple_GET_ITEM(args, 3)); /* name */
 #  endif
 
   ShaderCreateInfo *info = reinterpret_cast<ShaderCreateInfo *>(self->info);
   info->image(slot,
-              (eGPUTextureFormat)pygpu_texformat.value_found,
+              (blender::gpu::TextureFormat)pygpu_texformat.value_found,
               qualifier,
               blender::gpu::shader::ImageReadWriteType(pygpu_imagetype.value_found),
               name);
@@ -1219,7 +1232,7 @@ PyDoc_STRVAR(
     "   :type z: int\n");
 static PyObject *pygpu_shader_info_local_group_size(BPyGPUShaderCreateInfo *self, PyObject *args)
 {
-  int x = -1, y = -1, z = -1;
+  int x = -1, y = 1, z = 1;
 
   if (!PyArg_ParseTuple(args, "i|ii:local_group_size", &x, &y, &z)) {
     return nullptr;
@@ -1360,7 +1373,6 @@ PyDoc_STRVAR(
     ".. class:: GPUShaderCreateInfo()\n"
     "\n"
     "   Stores and describes types and variables that are used in shader sources.\n");
-
 PyTypeObject BPyGPUShaderCreateInfo_Type = {
     /*ob_base*/ PyVarObject_HEAD_INIT(nullptr, 0)
     /*tp_name*/ "GPUShaderCreateInfo",
