@@ -48,7 +48,7 @@
 #include "wm_xr_intern.hh"
 
 static wmSurface *g_xr_surface = nullptr;
-static CLG_LogRef LOG = {"wm.xr"};
+static CLG_LogRef LOG = {"xr"};
 
 /* -------------------------------------------------------------------- */
 
@@ -71,6 +71,10 @@ static void wm_xr_session_create_cb()
     settings->base_scale = 1.0f;
   }
   state->prev_base_scale = settings->base_scale;
+
+  /* Initialize vignette. */
+  state->vignette_data = MEM_callocN<wmXrVignetteData>(__func__);
+  WM_xr_session_state_vignette_reset(state);
 }
 
 static void wm_xr_session_controller_data_free(wmXrSessionState *state)
@@ -84,9 +88,27 @@ static void wm_xr_session_controller_data_free(wmXrSessionState *state)
   }
 }
 
+static void wm_xr_session_vignette_data_free(wmXrSessionState *state)
+{
+  if (state->vignette_data) {
+    MEM_freeN(state->vignette_data);
+    state->vignette_data = nullptr;
+  }
+}
+
+static void wm_xr_session_raycast_model_free(wmXrSessionState *state)
+{
+  if (state->raycast_model) {
+    GPU_batch_discard(state->raycast_model);
+    state->raycast_model = nullptr;
+  }
+}
+
 void wm_xr_session_data_free(wmXrSessionState *state)
 {
   wm_xr_session_controller_data_free(state);
+  wm_xr_session_vignette_data_free(state);
+  wm_xr_session_raycast_model_free(state);
 }
 
 static void wm_xr_session_exit_cb(void *customdata)
@@ -384,6 +406,8 @@ void wm_xr_session_state_update(const XrSessionSettings *settings,
   state->is_view_data_set = true;
   /* Assume this was already done through wm_xr_session_draw_data_update(). */
   state->force_reset_to_base_pose = false;
+
+  WM_xr_session_state_vignette_update(state);
 }
 
 wmXrSessionState *WM_xr_session_state_handle_get(const wmXrData *xr)
@@ -572,6 +596,52 @@ void WM_xr_session_state_navigation_reset(wmXrSessionState *state)
   unit_qt(state->nav_pose.orientation_quat);
   state->nav_scale = 1.0f;
   state->is_navigation_dirty = true;
+  state->swap_hands = false;
+}
+
+void WM_xr_session_state_vignette_reset(wmXrSessionState *state)
+{
+  wmXrVignetteData *data = state->vignette_data;
+
+  /* Reset vignette state */
+  data->aperture = 1.0f;
+  data->aperture_velocity = 0.0f;
+
+  /* Set default vignette parameters */
+  data->initial_aperture = 0.25f;
+  data->initial_aperture_velocity = -0.03f;
+
+  data->aperture_min = 0.08f;
+  data->aperture_max = 0.3f;
+
+  data->aperture_velocity_max = 0.002f;
+  data->aperture_velocity_delta = 0.01f;
+}
+
+void WM_xr_session_state_vignette_activate(wmXrData *xr)
+{
+  if (WM_xr_session_exists(xr)) {
+    wmXrVignetteData *data = xr->runtime->session_state.vignette_data;
+    data->aperture_velocity = data->initial_aperture_velocity;
+    data->aperture = min_ff(data->aperture, data->initial_aperture);
+  }
+}
+
+void WM_xr_session_state_vignette_update(wmXrSessionState *state)
+{
+  wmXrVignetteData *data = state->vignette_data;
+
+  const float vignette_intensity = U.xr_navigation.vignette_intensity;
+  const float aperture_min = interpf(
+      data->aperture_min, data->aperture_max, vignette_intensity * 0.01f);
+  data->aperture_velocity = min_ff(data->aperture_velocity_max,
+                                   data->aperture_velocity + data->aperture_velocity_delta);
+
+  if (data->aperture == aperture_min) {
+    data->aperture_velocity = data->aperture_velocity_max;
+  }
+
+  data->aperture = clamp_f(data->aperture + data->aperture_velocity, aperture_min, 1.0f);
 }
 
 /* -------------------------------------------------------------------- */
@@ -871,8 +941,8 @@ static void wm_xr_session_action_states_interpret(wmXrData *xr,
       break;
     }
     case XR_VECTOR2F_INPUT: {
-      const float(*state)[2] = &((float(*)[2])action->states)[subaction_idx];
-      float(*state_prev)[2] = &((float(*)[2])action->states_prev)[subaction_idx];
+      const float (*state)[2] = &((float (*)[2])action->states)[subaction_idx];
+      float (*state_prev)[2] = &((float (*)[2])action->states_prev)[subaction_idx];
       if (test_vec2f_state(
               *state, action->float_thresholds[subaction_idx], action->axis_flags[subaction_idx]))
       {
@@ -1009,7 +1079,7 @@ static bool wm_xr_session_action_test_bimanual(const wmXrSessionState *session_s
       break;
     }
     case XR_VECTOR2F_INPUT: {
-      const float(*state)[2] = &((float(*)[2])action->states)[*r_subaction_idx_other];
+      const float (*state)[2] = &((float (*)[2])action->states)[*r_subaction_idx_other];
       if (test_vec2f_state(*state,
                            action->float_thresholds[*r_subaction_idx_other],
                            action->axis_flags[*r_subaction_idx_other]))
@@ -1064,9 +1134,9 @@ static wmXrActionData *wm_xr_session_event_create(const char *action_set_name,
       data->float_threshold = action->float_thresholds[subaction_idx];
       break;
     case XR_VECTOR2F_INPUT:
-      copy_v2_v2(data->state, ((float(*)[2])action->states)[subaction_idx]);
+      copy_v2_v2(data->state, ((float (*)[2])action->states)[subaction_idx]);
       if (bimanual) {
-        copy_v2_v2(data->state_other, ((float(*)[2])action->states)[subaction_idx_other]);
+        copy_v2_v2(data->state_other, ((float (*)[2])action->states)[subaction_idx_other]);
       }
       data->float_threshold = action->float_thresholds[subaction_idx];
       break;
@@ -1390,23 +1460,23 @@ bool wm_xr_session_surface_offscreen_ensure(wmXrSurfaceData *surface_data,
   bool failure = false;
 
   /* Initialize with some unsupported format to check following switch statement. */
-  eGPUTextureFormat format = GPU_R8;
+  blender::gpu::TextureFormat format = blender::gpu::TextureFormat::UNORM_8;
 
   switch (draw_view->swapchain_format) {
     case GHOST_kXrSwapchainFormatRGBA8:
-      format = GPU_RGBA8;
+      format = blender::gpu::TextureFormat::UNORM_8_8_8_8;
       break;
     case GHOST_kXrSwapchainFormatRGBA16:
-      format = GPU_RGBA16;
+      format = blender::gpu::TextureFormat::UNORM_16_16_16_16;
       break;
     case GHOST_kXrSwapchainFormatRGBA16F:
-      format = GPU_RGBA16F;
+      format = blender::gpu::TextureFormat::SFLOAT_16_16_16_16;
       break;
     case GHOST_kXrSwapchainFormatRGB10_A2:
-      format = GPU_RGB10_A2;
+      format = blender::gpu::TextureFormat::UNORM_10_10_10_2;
       break;
   }
-  BLI_assert(format != GPU_R8);
+  BLI_assert(format != blender::gpu::TextureFormat::UNORM_8);
 
   offscreen = vp->offscreen = GPU_offscreen_create(draw_view->width,
                                                    draw_view->height,
